#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================


import os
import sys
import re
import argparse

import gcMapExplorer.lib as gmlib

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

description = \
"""Import HOMER Hi-C interaction matrix
=================================================

HOMER package(http://homer.salk.edu/homer/interactions/) contains modules to
analyze genome wide interaction data. It creates Hi-C matrix in a specific
format as shown as shown here:
http://homer.salk.edu/homer/interactions/HiCmatrices.html.

This format contains contact map in a matrix format.


=================================================
"""

inputFileHelp = \
""" File containing HOMER Hi-C ineraction matrix format contact map

"""

ccmapSuffixHelp = \
""" Use this to convert all contact maps to ccmaps file. Provide suffix of
ccmap file names with this option and it will enable the conversion.

Ouput ccmap file name is generated outmatically as follows;
<chromosome>_<resolution>_<suffix>.ccmap

Note that -od/--out-dir option is also required because all ccmaps will be
saved in this directory.

"""

outDirHelp = \
""" Directory to save all ccmap files. It should be provided when -ccma/--ccmap
option is used.

"""

fileGCMapHelp = \
"""Provide gcmap file to convert all contact maps into one gcmap file.
File name should contain full path because -od/--out-dir is not considered
for thi conversion.

"""

downsampleMethodHelp = \
"""Downsampling method to coarsen the resolution in gcmap file. The option is
intended to use with -gcm/--gcmap option. Three accepted methods are
        sum  : sum of values,
        mean : Average of values and
        max  : Maximum of values.

This option generates all coarser maps where resolutions will be coarsened by
a factor of two, consequetively. e.g.: In case of 10 kb input resolution,
downsampled maps of "20kb", "40kb", "80kb", "160kb", "320kb" etc. will be
generated until, map size is less than 500.

"""

def main():

    # Construct command line arguments and parsed it
    parser, args = parseArguments()

    # Input File
    inputFile = None
    if args.inputFile is not None:
        inputFile = args.inputFile.name
        args.inputFile.close()
    else:
        showErrorAndExit(parser, "No Input File!!!\n")

    if args.ccmapSuffix is not None and args.outDir is None:
        msg = "No output directory is given for ccmap files!!!\n"
        showErrorAndExit(parser, msg)

    if args.ccmapSuffix is None and args.fileGCMap is None:
        showErrorAndExit(parser, "No output format directed!!!\n")

    # Check for scratch directory
    if not os.path.isdir(args.workDir):
        showErrorAndExit(parser, '\nScratch Dirctory "{0}" not found !!!\n'.format(args.workDir))

    # Convert ccmaps
    if args.ccmapSuffix is not None:
        homer_reader = gmlib.importer.HomerInputHandler(inputFile,
                                                        workDir=args.workDir)
        homer_reader.save_ccmaps(args.outDir, suffix=args.ccmapSuffix)

        del homer_reader

    # Convert gcmap
    if args.fileGCMap is not None:
        homer_reader = gmlib.importer.HomerInputHandler(inputFile,
                                                        workDir=args.workDir)
        homer_reader.save_gcmap(args.fileGCMap,
                                coarsingMethod=args.coarsingMethod,
                                compression=args.compression)
        del homer_reader

def parseArguments():
    parser = argparse.ArgumentParser(
                prog='gcMapExplorer homer2cmap',
                description=description,
                formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-i', '--input', action='store',
                        type=argparse.FileType('r'), metavar='matrix.txt',
                        dest='inputFile', help=inputFileHelp)

    parser.add_argument('-ccm', '--ccmap', action='store', dest='ccmapSuffix',
                        metavar='RawObserved', help=ccmapSuffixHelp)

    parser.add_argument('-od', '--out-dir', action='store', dest='outDir',
                        help='Directory where all ccmap files will be saved.\n')

    parser.add_argument('-gcm', '--gcmap', action='store', dest='fileGCMap',
                        metavar='inOut.gcmap', help=fileGCMapHelp)

    parser.add_argument('-cmeth', '--compression-method', action='store',
                        dest='compression', metavar='lzf',
                        choices=['lzf', 'gzip'], default='lzf',
                        help='Data compression method for gcmap file.\n')

    parser.add_argument('-dmeth', '--downsample-method', action='store',
                        dest='coarsingMethod', metavar='sum',
                        choices=['max', 'mean', 'sum'], default='sum',
                        help=downsampleMethodHelp)

    parser.add_argument('-wd', '--work-dir', action='store', dest='workDir',
                        default=config['Dirs']['WorkingDirectory'],
                        metavar=config['Dirs']['WorkingDirectory'],
                        help='Directory where temporary files will be stored.')

    idx = sys.argv.index("homer2cmap")+1
    args = parser.parse_args(args=sys.argv[idx:])

    return parser, args


def checkFileExist(filename, parser):
    if not os.path.isfile(filename):
        parser.print_help()
        print("\n===== ERROR =======")
        print("File: {0} does not exist !!!\n".format(filename))
        print("See Usage Above!!!")
        sys.exit(-1)


def showErrorAndExit(parser, message):
    parser.print_help()
    print("\n===== ERROR =======")
    print(message)
    print("See Usage Above!!!")
    sys.exit(-1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np


from . import ccmapHelpers as cmh


def performIterativeCorrection(matrix, tol, iteration):
    count = 0
    B = np.ones(matrix.shape[0], dtype=np.float)

    while True:
        dB = np.sum(matrix, axis=0)              # Sum over rows
        dB = dB/np.mean(dB[np.nonzero(dB != 0)])      # Renormalization by mean value
        dB[np.nonzero(dB == 0)] = 1                   # Set zeros value to one

        # Divide W_ij by dB_i dB_j
        for i in range(matrix.shape[0]):
            matrix[i] = matrix[i] / dB
            matrix[:, i] = matrix[:, i] / dB

        B = B *  dB

        if np.var(dB) < tol or count > iteration:
            break

        count = count + 1



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
from multiprocessing import Process
import re, sys, os, shutil
import json
import logging

from . import ccmapHelpers as cmh
from . import ccmap as cmp


def remove_by_scores(peaks, ccmap):
	boxes = dict()
	c = 0
	for p in peaks:
		box = TADBOX(p)
		boxes[c] = box
		c += 1

	for i in range(len(boxes)):
		if boxes[i] is None:
			continue
		si0 = boxes[i].startingPoint[0]
		si1 = boxes[i].startingPoint[1]
		ei0 = boxes[i].endPoint[0]
		ei1 = boxes[i].endPoint[1]
		for j in range(i):
			if boxes[j] is None:
				continue
			sj0 = boxes[j].startingPoint[0]
			sj1 = boxes[j].startingPoint[1]
			ej0 = boxes[j].endPoint[0]
			ej1 = boxes[j].endPoint[1]

			if (abs(si0 - sj0) <=4 and abs(si1 - sj1) <=4) or (abs(ei0 - ej0) <=4 and abs(ei1 - ej1) <=4) :
				if abs(boxes[i].get_score(ccmap, 1.0) - boxes[j].get_score(ccmap, 1.0)) <= 0.05:
					if boxes[i].size > boxes[j].size:
						boxes[j] = None
					else:
						boxes[i] = None
						break

	new_peaks = []
	scores = []
	for key in boxes:
		if boxes[key] is not None:
			new_peaks.append(boxes[key].Vertex)
			scores.append(boxes[key].get_score(ccmap, 1.0))

	idx = np.argsort(scores)
	percent = int(len(idx)*50/100)

	return np.asarray(new_peaks)


def remove_peaks_from_diagonals(peaks, includeCells=4):
	new_peaks = []
	for p in peaks:
		if abs(p[0] - p[1]) <= includeCells:
			continue
		if p[0] > p[1]:
			continue
		new_peaks.append(p)
	return np.asarray(new_peaks)

class TADBOX(object):
	def __init__(self, vertex):
		self.Vertex = vertex
		self.size = None
		self.startingPoint = None
		self.endPoint = None
		self.set_start_end_and_size()
		self.edgeIndex = []
		self.get_edge_index()
		self.midPoint = self.get_mid_point()
		self.score = None

	def get_edge_index(self):
		iVertex = self.startingPoint[0] + self.size
		jVertex = self.startingPoint[1] + self.size
		for i in range(self.startingPoint[0], iVertex):
			if abs(i - jVertex) <= 4:
				continue
			self.edgeIndex.append([i, jVertex])

		for j in range(self.startingPoint[1], jVertex):
			if abs(j - iVertex) <= 4:
				continue
			self.edgeIndex.append([iVertex, j])

		self.edgeIndex = np.asarray(self.edgeIndex).T

	def set_start_end_and_size(self):
		minp = min(self.Vertex[0], self.Vertex[1])
		maxp = max(self.Vertex[0], self.Vertex[1])
		self.startingPoint = (minp, minp)
		self.endPoint = (maxp, maxp)
		self.size = maxp - minp + 1

	def get_mid_point(self):
		i = int ( (self.startingPoint[0] + self.size)/2 )
		self.midPoint = [ i, i ]

	def calc_score(self, ccmap, pfactor):
		self.score = -1 * pfactor * np.log(ccmap.matrix[self.edgeIndex]).mean()

	def get_score(self, ccmap, pfactor):
		if self.score is None:
			self.calc_score(ccmap, pfactor)
		return self.score

	def copy(self, del_index=False):
		box = TADBOX(self.startingPoint, self.size)
		if del_index:
			box.edgeIndex = None
		return box

'''
class TADFINDER(object):
	def __init__(self, ccmap, pfactor=1, mc_steps=100000, minsize=100, maxsize=1e5, expand=1000):
		self.hicimap = ccmap
		self.check_input_box_size(minsize, maxsize, expand)
		self.OutPutBoxes = None
		self.minBoxSize = minsize
		self.maxBoxSize = maxsize
		self.expansion = 1000
		self.boxsize = minsize
		self.moving_box = None
		self.pfactor = pfactor
		self.mc_steps = mc_steps

	def check_input_box_size(self, minsize, maxsize, expand):
		if self.hicimap.shape[0] < maxsize:
			raise ValueError ('Input maximum box size is larger than HiC-map size.')

		if self.hicimap.shape[0] < minsize:
			raise ValueError ('Input minimum box size is larger than HiC-map size.')


	def minimize_box_shape_local(self):
		""" Find a box with minimum energy at given starting point for a given box size
		"""
		old_score = self.moving_box.get_score(self.hicimap, self.pfactor)
		new_box = self.moving_box
		for i in range(1, self.expansion):
			if self.moving_box.size + i >= self.ccmap.shape[0]:
				break
			temp_box = TADBOX(self.moving_box.startingPoint, self.moving_box.size + i)
			new_score = temp_box.get_score(ccmap, self.pfactor)
			if new_score < old_score:
				new_box = temp_box
				oldscore = new_score
		self.OutPutBoxes.append(new_box.cop(del_index=True))

	def move_box(self):
		"""	Move box according to Metropolis Monte Carlo for a given box size
		"""
		if moving_box is not None:
			old_score = self.moving_box.get_score(self.hicimap, self.pfactor)
		else:
			old_score = 100000

		s = np.random.random_integers(int(self.boxsize/2), high=self.hicimap.shape[0]-int(self.boxsize/2))

		new_box = TADBOX((s, s), self.boxsize)
		new_score = new_box.get_score(self.hicimap, self.pfactor)
		moved = False
		if new_score < old_score:
			self.moving_box = new_box
			moved = True
		else:
			p = np.exp( -1 * self.pfactor * (new_score - old_score) )
			if p > random.random_sample():
				self.moving_box = new_box
				moved = True


	def do_MC_sampling(self):
		""" Do monte carlo sampling for the given box size
		"""
		for i in range(self.steps):
			if self.move_box():
				self.minimize_box_shape_local()

	def find(self, forSize=None):
		""" Do monte carlo sampling for all box sizes
		"""
		if forSize is None:
			for b in range(self.minBoxSize, self.maxBoxSize, self.expansion):
				self.do_MC_sampling()
				self.boxsize = b + self.expansion
		else:
			self.boxsize = forSize
			self.do_MC_sampling()

	def remove_duplicates(self):
		new_boxes = []
		for i in range(len(self.OutPutBoxes)):
			if i == 0:
				new_boxes.append(self.OutPutBoxes[i])
			else:
				for b in new_boxes:
					if b.startingPoint == self.OutPutBoxes[i].startingPoint and b.size == self.OutPutBoxes[i].size:
						continue
					else:
						new_boxes.append(self.OutPutBoxes[i])
						break

		self.OutPutBoxes = new_boxes

	def write_output_file(self, filename):
		fout = open(filename, 'w')
		for b in self.OutPutBoxes:
			fout.write('{0} {1} {2}' .format(b.startingPoint[0], b.size, np.exp( -1 * (b.score/self.pfactor) )))

	def run(self, outfile, forSize=None):
		self.find(forSize)
		self.remove_duplicates()
		self.write_output_file(outfile)
'''

from math import ceil
from multiprocessing import cpu_count

def _get_chunks(shape, ncpu):
	"""Split the array into equal sized chunks based on the number of
	available processors. The last chunk in each dimension absorbs the
	remainder array elements if the number of CPUs does not divide evenly into
	the number of array elements.

	Examples
	--------
	>>> _get_chunks((4, 4), 4)
	((2, 2), (2, 2))
	>>> _get_chunks((4, 4), 2)
	((2, 2), (4,))
	>>> _get_chunks((5, 5), 2)
	((2, 3), (5,))
	>>> _get_chunks((2, 4), 2)
	((1, 1), (4,))
	"""
	chunks = []
	nchunks_per_dim = int(ceil(ncpu ** (1./len(shape))))

	used_chunks = 1
	for i in shape:
		if used_chunks < ncpu:
			regular_chunk = i // nchunks_per_dim
			remainder_chunk = regular_chunk + (i % nchunks_per_dim)

			if regular_chunk == 0:
				chunk_lens = (remainder_chunk,)
			else:
				chunk_lens = ((regular_chunk,) * (nchunks_per_dim - 1) + (remainder_chunk,))
		else:
			chunk_lens = (i,)

		chunks.append(chunk_lens)
		used_chunks *= nchunks_per_dim
	return tuple(chunks)


def apply_parallel(function, array, chunks=None, depth=0, mode=None, extra_arguments=(), extra_keywords={}):
	"""Map a function in parallel across an array.

	Split an array into possibly overlapping chunks of a given depth and
	boundary type, call the given function in parallel on the chunks, combine
	the chunks and return the resulting array.

	Parameters
	----------
	function : function
	    Function to be mapped which takes an array as an argument.
	array : numpy array
	    Array which the function will be applied to.
	chunks : int, tuple, or tuple of tuples, optional
	    A single integer is interpreted as the length of one side of a square
	    chunk that should be tiled across the array.  One tuple of length
	    ``array.ndim`` represents the shape of a chunk, and it is tiled across
	    the array.  A list of tuples of length ``ndim``, where each sub-tuple
	    is a sequence of chunk sizes along the corresponding dimension. If
	    None, the array is broken up into chunks based on the number of
	    available cpus. More information about chunks is in the documentation
	    `here <https://dask.pydata.org/en/latest/array-design.html>`_.
	depth : int, optional
	    Integer equal to the depth of the added boundary cells. Defaults to
	    zero.
	mode : {'reflect', 'symmetric', 'periodic', 'wrap', 'nearest', 'edge'}, optional
	    type of external boundary padding.
	extra_arguments : tuple, optional
	    Tuple of arguments to be passed to the function.
	extra_keywords : dictionary, optional
	    Dictionary of keyword arguments to be passed to the function.

	Notes
	-----
	Numpy edge modes 'symmetric', 'wrap', and 'edge' are converted to the
	equivalent `dask` boundary modes 'reflect', 'periodic' and 'nearest',
	respectively.
	"""
	import dask.array as da

	if chunks is None:
		shape = array.shape
		try:
			ncpu = cpu_count()
		except NotImplementedError:
			ncpu = 4
		chunks = _get_chunks(shape, ncpu)

	if mode == 'wrap':
		mode = 'periodic'
	elif mode == 'symmetric':
		mode = 'reflect'
	elif mode == 'edge':
		mode = 'nearest'

	def wrapped_func(arr):
		return function(arr, *extra_arguments, **extra_keywords)

	darr = da.from_array(array, chunks=chunks)
	return darr.map_overlap(wrapped_func, depth, boundary=mode).compute()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
from scipy import stats as spstats
import re, os, sys
import copy
import shlex, subprocess
import tempfile
import logging
import urllib
import shutil
import h5py

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import ccmap as cmp


def check_resolution_list(resolutions):
    new_resolutions = [ '1kb', '2kb', '4kb', '5kb', '8kb', '10kb', '20kb', '40kb', '80kb', '100kb', '160kb', '200kb', '320kb', '500kb', '640kb', '1mb']

    if resolutions is not None:
        if isinstance(resolutions, list):
            new_resolutions = list( set(new_resolutions + resolutions) )
        else:
            raise ValueError(' Resolutions should be of list type.')

    return new_resolutions

class TempNumpyArrayFiles:
    """To handle temporary numpy array files

    To convert a Wig file to hdf5 file, data are parsed, and further stored temporarily in these memory-mapped numpy array files.
    Use of numpy arrays avoid dependency from storing chromosome location/coordinates because array index is used as the location/coordinates.
    Additionaly, chromosome could be very large and to store these arrays could be memory expensive, these arrays are stored as binary files on the disk.

    .. note::
        These generated files are either automatically deleted after execution of script or by deleting [``del``] :class:`TempNumpyArrayFiles` instance.

    Attributes
    ----------
    chromSizeInfo : dict
        Dictionary contains chromosome size. Numpy array files will be generated on the basis of these sizes.
    files : dict
        Dictionary for names of temporary numpy array files, where keys are chromosomes and values are respective file names.
    arrays : dict
        Dictionary of memory mapped numpy array (`numpy.memmap <http://docs.scipy.org/doc/numpy-1.10.0/reference/generated/numpy.memmap.html>`_), where
        keys are chromosomes and values are respective `numpy.memmap <http://docs.scipy.org/doc/numpy-1.10.0/reference/generated/numpy.memmap.html>`_ arrays.
    workDir : str
        Working directory where temporary numpy array files will be generated.



    """
    def __init__(self, workDir=None):
        self.chromSizeInfo = None
        self.files = None
        self.arrays = None

        # Working and output directory
        if workDir is None:
            self.workDir = config['Dirs']['WorkingDirectory']
        else:
            self.workDir = workDir

        self.logger = logging.getLogger(__name__+'.TempNumpyArrayFiles')
        self.logger.setLevel(logging.INFO)

    def __del__(self):
        self._removeTempNumpyFiles()

    def _removeTempNumpyFiles(self):
        if self.arrays is not None:
            for key in self.arrays:
                self._removeTempNumpyFile(key)

    def _removeTempNumpyFile(self, key):
        self.logger.info(' Closing and removing temporary numpy array file [{0}] .' .format(self.files[key]))
        try:
            self.arrays[key]._mmap.close()
        except:
            pass

        try:
            os.remove(self.files[key])
        except:
            pass

    def updateArraysByBigWig(self, bigWigFileName):
        """Update/resize all array files using given bigWig file

        Parameters
        ----------
        bigWigFileName : str
            Name of input bigWig file


        """
        chromSizeInfo = self._getBigWigInfo(bigWigFileName)
        for key in chromSizeInfo:
            if key in self.chromSizeInfo:
                if chromSizeInfo[key] > self.chromSizeInfo[key]:
                    self.chromSizeInfo[key] = chromSizeInfo[key]
                    self.generateTempNumpyFile(key, regenerate=True)
            else:
                self.chromSizeInfo[key] = chromSizeInfo[key]
                self.generateTempNumpyFile(key)

    def updateArraysByChromSize(self, chrom, size):
        """Update/resize an array file using given chromosome and its size

        Parameters
        ----------
        chrom : str
            Chromosome name
        size : int
            Total size of chromosome

        """
        if chrom in self.chromSizeInfo:
            if size > self.chromSizeInfo[chrom]:
                self.chromSizeInfo[chrom] = size
                self.generateTempNumpyFile(chrom, regenerate=True)
        else:
            self.chromSizeInfo[chrom] = size
            self.generateTempNumpyFile(chrom)


    def addChromSizeInfo(self, bigWigFileName):
        """ Update chromosome sizes using new bigWig file

        To update :attr:`TempNumpyArrayFiles.chromSizeInfo` for new bigWig files, this method can be used.

        .. note::
            This method only updates the :attr:`TempNumpyArrayFiles.chromSizeInfo` dictionary. It does not resize numpy array files.

        Parameters
        ----------
        bigWigFileName : str
            Name of input bigWig file

        """
        chromSizeInfo = self._getBigWigInfo(bigWigFileName)

        for key in chromSizeInfo:
            if key in self.chromSizeInfo:
                self.chromSizeInfo[key] = max( chromSizeInfo[key], self.chromSizeInfo[key] )
            else:
                self.chromSizeInfo[key] = chromSizeInfo[key]

    def _getBigWigInfo(self, filename):
        """Base method to Retrieve chromosome names and their sizes

        * Use :meth:`TempNumpyArrayFiles.addChromSizeInfo` to automatically retrieve chromosome size information from bigWig file and to store in :attr:`TempNumpyArrayFiles.chromSizeInfo`.

        .. warning::
            **Private method.** Use it at your own risk. It is used internally in :meth:`TempNumpyArrayFiles.addChromSizeInfo`

        Parameters
        ----------
        filename : str
            Input bigWig file

        """

        cmd = '{0} -chroms {1}' .format(self.pathTobigWigInfo, filename)

        args = shlex.split(cmd)
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        self.logger.info(' Started running command: [{0}] ...' .format(cmd))
        stdout, stderr = p.communicate()
        if p.returncode != 0:
            self.logger.info(stdout)
            self.logger.info(stderr)
            self.logger.error(' Not able to run command: [ {0} ]. See details above.' .format(cmd))
        self.logger.info('         ... Finished!!!' .format(cmd))

        chromSizeInfo = dict()

        chroms = None
        read = False
        counter = 0
        for line in re.split('\n', stdout.decode("utf-8")):
            if re.search('chromCount', line) != None:
                temp = re.split('\s+', line)
                chroms = int(temp[1])
                read = True
                continue

            if read:
                line = line.lstrip().rstrip()
                temp = re.split('\s+', line)
                chromSizeInfo[temp[0]] = int(temp[2])
                counter += 1

                if counter >= chroms:
                    read = False

        return chromSizeInfo

    def genrateAllTempNumpyFiles(self):
        """Generate all memory mapped numpy array files

        It is used to generate all memory mapped numpy array files using the :attr:`TempNumpyArrayFiles.chromSizeInfo` dictionary.

        """

        if self.self.chromSizeInfo is None:
            raise ValueError(" Chromosome sizes are not available. ")
        if self.files is None:
            self.files = dict()

        self.logger.info(' Generating temporary numpy array files...')
        for key in self.chromSizeInfo:
            self.generateTempNumpyFile(key)

    def generateTempNumpyFile(self, key, regenerate=False):
        """Generate a memory mapped numpy array file

        It is used to generate a memory mapped numpy array for given chromosome for which size is already the :attr:`TempNumpyArrayFiles.chromSizeInfo` dictionary.

        Parameters
        ----------
        key : str
            chromosome name. Should be present as key in :attr:`TempNumpyArrayFiles.chromSizeInfo`.
        regenerate : bool
            Replace or regenerate new memory mapped array for given chromosome

        """

        try:
            self._generateTempNumpyFile(key, regenerate=regenerate)
        except (SystemExit, KeyboardInterrupt) as e:
            self._removeTempNumpyFiles()
            raise e

    def _generateTempNumpyFile(self, key, regenerate=False):
        """enerate a memory mapped numpy array file

        It is used to generate a memory mapped numpy array file for given chromosome for which size is already in the :attr:`TempNumpyArrayFiles.chromSizeInfo` dictionary.

        .. warning::
            **Private method.** Use it at your own risk. It is used internally in :meth:`TempNumpyArrayFiles.generateTempNumpyFile`.

        Parameters
        ----------
        key : str
            chromosome name. Should be present as key in :attr:`TempNumpyArrayFiles.chromSizeInfo`.
        regenerate : bool
            Replace or regenerate new memory mapped array for given chromosome

        """
        if self.files is None:
            self.files = dict()

        if self.arrays is None:
            self.arrays = dict()

        if (key in self.files) and not regenerate:
            self.logger.info('{0} is already present in temporary numpy file list, skipping ...' .format(key))
        else:
            if regenerate:
                self._removeTempNumpyFile(key)

            (fd, fname) = tempfile.mkstemp(suffix='.npy', prefix=key+'_', dir=self.workDir, text=False)
            os.close(fd)     # Close file, error in windows OS
            self.logger.info(' Generating temporary numpy array file [{0}] for {1} ...' .format(fname, key))

            size = self.chromSizeInfo[key] + 1        # Be careful: added one to easiliy handle real locations, zeroth index is dummy, dont use zeroth location
            np.save(fname, np.zeros(size, dtype=np.float))
            npy = np.load(fname, mmap_mode='r+')

            self.arrays[key] = npy
            self.files[key] = fname

            # print(npy.shape)
            self.logger.info(' \t... Finished generating temporary numpy array file.')

    def fillAllArraysWithZeros(self):
        """Fill all arrays with zeros.

        To fill all memory mapped array with zero. It is used in :class:`WigHandler` so that new data extracted from Wig files can be stored in these array files.

        """
        if self.arrays is not None:
            for key in self.chromSizeInfo:
                self.arrays[key].fill(0.0)

class HDF5Handler:
    """Handler for genomic data HDF5 file.

    This class acts like a handler and can be used to read, write, and modify genomic data file in `HDF5 format <https://www.hdfgroup.org/why_hdf/>`_.
    This is a binary file and is compressed using ``zlib`` method to reduce the storage memory.


    Structure of HDF5 file: ``/<Chromosome>/<Resolution>/<1D Numpy Array>``

        ::

          HDF5 ──────────────────────────> title
            ├──────── chr1
            │           ├───── 1kb
            │           │        ├──────── amean  ( Arithmatic mean) (type: 1D Numpy Array)
            │           │        ├──────── median ( Median value   ) (type: 1D Numpy Array)
            │           │        ├──────── hmean  ( Harmonic mean  ) (type: 1D Numpy Array)
            │           │        ├──────── gmean  ( Geometric mean ) (type: 1D Numpy Array)
            │           │        ├──────── min    ( Minimum value  ) (type: 1D Numpy Array)
            │           │        └──────── max    ( Maximum value  ) (type: 1D Numpy Array)
            │           │
            │           ├────  5kb
            │           │        ├──────── amean  ( Arithmatic mean) (type: 1D Numpy Array)
            │           │        ├──────── median ( Median value   ) (type: 1D Numpy Array)
            │           │        ├──────── hmean  ( Harmonic mean  ) (type: 1D Numpy Array)
            │           │        ├──────── gmean  ( Geometric mean ) (type: 1D Numpy Array)
            │           │        ├──────── min    ( Minimum value  ) (type: 1D Numpy Array)
            │           │        └──────── max    ( Maximum value  ) (type: 1D Numpy Array)
            │           │
            │           └────  ...
            │
            ├──────── chr2
            │           ├───── 1kb
            │           │        ├──────── amean  ( Arithmatic mean) (type: 1D Numpy Array)
            │           │        ├──────── median ( Median value   ) (type: 1D Numpy Array)
            │           │        ├──────── hmean  ( Harmonic mean  ) (type: 1D Numpy Array)
            │           │        ├──────── gmean  ( Geometric mean ) (type: 1D Numpy Array)
            │           │        ├──────── min    ( Minimum value  ) (type: 1D Numpy Array)
            │           │        └──────── max    ( Maximum value  ) (type: 1D Numpy Array)
            │           └────  ..
            :
            :
            :
            └───── ...


    Attributes
    ----------
    filename : str
        HDF5 file name

    title : str
        Title of the data

    hdf5 : h5py.File
        input/output stream to HDF5 file

    data : dict
        This dictionary is generated by :meth:`HDF5Handler.buildDataTree()`. This dictionary gives access to all data arrays.



    Parameters
    ----------
    filename : str
        HDF5 file name. e.g.: ``abcxyz.h5``
    title : str
        title or name of the data



    Examples
    --------
    .. code-block:: python

        from hiCMapAnalyze import genomicsDataHandler as gdh
        import numpy as np

        # Load available file
        hdf5Hand = gdh.HDF5Handler('test.h5')

        # Build the data structure, this is an essential step to retrieve the data easiliy as shown below
        hdf5Hand.buildDataTree()

        # Print shape and maximum value of chr1->1kb->mean array
        print(hdf5Hand.data['chr1']['1kb']['mean'].shape, np.amax(hdf5Hand.data['chr1']['1kb']['mean']))


    """

    def __init__(self, filename, title=None):
        self.logger = logging.getLogger(__name__+'.HDF5Handler')
        self.logger.setLevel(logging.INFO)

        self.filename = filename
        self.hdf5 = None

        self.title = None
        if title is not None:
            self.title = title

        self.data = None

    def __del__(self):
        if self.hdf5 is not None:
            self.close()

    def close(self):
        """ close hdf5 file
        """
        try:
            self.hdf5.close()
            self.logger.info(' Closed {0} ...' .format(self.filename))
            self.hdf5 = None
        except:
            pass

    def open(self):
        """ open hdf5 file
        """
        self.hdf5 = h5py.File(self.filename)

        # Fail-safe mechanism for title in case of either new file or already opened file
        if 'title' not in self.hdf5.attrs:
            if self.title is None:
                self.hdf5.attrs['title'] = "Genome Data"
                self.title = "Genome Data"
            else:
                self.hdf5.attrs['title'] = self.title
        else:
            if self.title is None:
                self.title = self.hdf5.attrs['title']

        self.logger.info(' Opened {0} ...' .format(self.filename))

    def getChromList(self):
        """To get list of all chromosomes present in hdf5 file

        Returns
        -------
        chroms : list
            List of all chromosomes present in hdf5 file

        """
        if self.hdf5 is None:
            self.open()

        chroms = []
        for g in self.hdf5.keys():
            chroms.append(g)

        return chroms

    def getResolutionList(self, chrom):
        """ To get all resolutions for given chromosome from hdf5 file

        Parameters
        ----------
        chrom : str
            chromosome name

        Returns
        -------
        resolutionList : list[str]
            A list of all available resolutions for the given chromosome

        Raises
        ------
        KeyError
            If chromosome not found in hdf5 file

        """
        chromList = self.getChromList()

        resolutionList = []
        if chrom in chromList:
            for r in self.hdf5[chrom].keys():
                resolutionList.append(r)
        else:
            raise KeyError(' Chromosome [{0}] not found in [{1}] file...' .format(chrom, self.filename))

        return resolutionList

    def getDataNameList(self, chrom, resolution):
        """ List of all available arrays by respecitve coarse method name for given chromosome and resolution

        Parameters
        ----------
        chrom : str
            chromosome name
        resolution : str
            resolution

        Returns
        -------
        nameList : list[str]
            List of arrays by name of dataset

        Raises
        ------
        KeyError
            If chromosome not found in hdf5 file. If input resolution keyword is not found for input chromosome.

        """

        nameList = []

        if chrom in self.getChromList():
            if resolution in self.getResolutionList(chrom):
                for m in self.hdf5[chrom][resolution].keys():
                    nameList.append(m)
            else:
                raise KeyError(' Resolution [{0}] for Chromosome [{1}] not found in [{2}] file...' .format(resolution, chrom, self.filename))

        else:
            raise KeyError(' Chromosome [{0}] not found in [{1}] file...' .format(chrom, self.filename))

        return nameList

    def buildDataTree(self):
        """ Build data dictionary from the input hdf5 file

        To retrieve the data from hdf5 file, this function should be used to built the dictionary :attr:`HDF5Handler.data`.
        This dictionary gives access directly to data of any chromosome with specific resolution.

        """

        if self.data is None:
            self.data = dict()
        for chrom in self.getChromList():
            self.data[chrom] = dict()
            for resolution in self.getResolutionList(chrom):
                self.data[chrom][resolution] = dict()
                for cm in self.getDataNameList(chrom, resolution):
                    self.data[chrom][resolution][cm] = self.hdf5[chrom][resolution][cm]

    def addDataByArray(self, Chrom, resolution, data_name, value_array, compression='lzf'):
        """ Add array to the hdf5 file for given chromosome, resolution and data name.
            It can be used either to add new data array or to replace existing data.

        Parameters
        ----------
        Chrom : str
            Chromosome Name
        resolution : str
            Reslution of data
        data_name : str
            Name of data.
        value_array : numpy.ndarray
            An array containing values.

        """

        if self.hdf5 is None:
            self.open()

        firstLevel = '/' + Chrom
        secondLevel = '/' + Chrom + '/' + resolution
        thirdLevel = '/' + Chrom + '/' + resolution + '/' + data_name

        if Chrom not in self.hdf5:
            self.hdf5.create_group(Chrom)

        if resolution not in self.hdf5[Chrom]:
            self.hdf5[Chrom].create_group(resolution)

        array = np.asarray(value_array)
        if data_name not in self.hdf5[Chrom][resolution]:
            if compression == 'gzip':
                self.hdf5[Chrom][resolution].create_dataset(data_name, array.shape, dtype=array.dtype, data=array, compression="gzip", shuffle=True, compression_opts=4)
            else:
                self.hdf5[Chrom][resolution].create_dataset(data_name, array.shape, dtype=array.dtype, data=array, compression="lzf", shuffle=True)
        else:
            self.hdf5[Chrom][resolution].pop(data_name)
            if compression == 'gzip':
                self.hdf5[Chrom][resolution].create_dataset(data_name, array.shape, dtype=array.dtype, data=array, compression="gzip", shuffle=True, compression_opts=4)
            else:
                self.hdf5[Chrom][resolution].create_dataset(data_name, array.shape, dtype=array.dtype, data=array, compression="lzf", shuffle=True)

        self.hdf5[Chrom][resolution][data_name].attrs['compression'] = compression


class BigWigHandler:
    """ To handle bigWig files and to convert it to hdf5 file

    This class can be used to convert bigWig file to hdf5 file.
    It can also be used to combine several bigWig files that are originated from replicated experiments.

    Attributes
    ----------
    bigWigFileNames : str or list[str]
        List of bigWig file names including path
    pathTobigWigToWig : str
        Path to bigWigToWig program
    pathTobigWigInfo : str
        Path to bigWigInfo program
    WigFileNames : str
        List of Wig file names, either autmoatically generated or given by user
    wigHandle : WigHandler
        WigHandler instance to parse Wig file and save data as hdf5 file
    chromSizeInfo : dict
        A dictionary containing chromosome size information
    methodToCombine : str
        method to combine bigWig/Wig files, Presently, accepted keywords are: ``mean``, ``min`` and ``max``
    chromName : str
        Name of chromosome, which has to be extracted from bigWig file.
    resolutions : list[str]
        List of input resolutions for which data will be stored in the hdf5 file
    maxEntryWrite : int
        Number of lines read from Wig file at an instant, after this, data is dumped in temporary numpy array file

    Parameters
    ----------
    filenames : str or list[str]
        A bigWig file or list of bigWig files including path
    pathTobigWigToWig: str
        Path to bigWigToWig program
    pathTobigWigInfo : str
        Path to bigWigInfo program
    resolutions : list[str] (optional)
        List of input resolutions for which data will be stored inside hdf5 file. If its value is ``None``, a new default list is generated by :meth:`check_resolution_list`
    methodToCombine : str
        method to combine bigWig/Wig files, Presently, accepted keywords are: ``mean``, ``min`` and ``max``
    chromName : str
        Name of chromosome, which has to be extracted from bigWig file.
    maxEntryWrite : int
        Number of lines read from Wig file at an instant, after this, data is dumped in temporary numpy array file. To reduce memory (RAM) occupancy,
        reduce this number because large numbers need large RAM.

    """

    def __init__(self, filenames, pathTobigWigToWig, pathTobigWigInfo, resolutions=None, chromName=None, methodToCombine='mean', workDir=None, maxEntryWrite=10000000):
        self.bigWigFileNames = filenames
        self.pathTobigWigToWig = pathTobigWigToWig
        self.pathTobigWigInfo = pathTobigWigInfo
        self.chromName = chromName            # If only an input chromosome is required

        self.WigFileNames = None           # Wig file name
        self.wigToDel = False             # Flag to delete or retain wig file
        self.wigHandle = None             # Used for WigHandler object

        self.chromSizeInfo = None         # dictionary for chromosome sizes, extracted from bigWigInfo program

        # Working and output directory
        if workDir is None:
            self.workDir = config['Dirs']['WorkingDirectory']
        else:
            self.workDir = workDir

        self.logger = logging.getLogger(__name__+'.BigWigHandler')
        self.logger.setLevel(logging.INFO)

        self.maxEntryWrite = maxEntryWrite # maximum count before writing to output file will be attempted

        # Convert to list
        if not isinstance(filenames, list):
            self.bigWigFileNames = [ filenames ]

        if not (methodToCombine == 'mean' or methodToCombine == 'max' or methodToCombine == 'min'):
            raise NotImplementedError(' Method [{0}] to combine bigwig file not implemented. Use: \'mean\', \'max\' or \'min\' ' .format(methodToCombine))
        self.methodToCombine = methodToCombine

        self.resolutions = check_resolution_list(resolutions)

    def __del__(self):
        self._removeTempWigFiles()

    def _removeTempWigFiles(self):
        if self.wigToDel:
            for f in self.WigFileNames:
                self.logger.info(' Removing temporary Wig file [{0}] .' .format(f))
                try:
                    os.remove(f)
                except:
                    pass

    def _getBigWigInfo(self, filename):
        """Base method to Retrieve chromosome names and their sizes

        * Chromosome size information is stored for a given bigWig file. If size of chromosome is already present in dictionary, largest size is stored in dictionary.

        * Use :meth:`BigWigHandler.getBigWigInfo` to automatically retrieve chromosome size information from all bigWig files.

        .. warning::
            **Private method.** Use it at your own risk. It is used internally in :meth:`BigWigHandler.getBigWigInfo`

        Parameters
        ----------
        filename : str
            Input bigWig file

        """
        cmd = '{0} -chroms {1}' .format(self.pathTobigWigInfo, filename)

        args = shlex.split(cmd)
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        self.logger.info(' Started running command: [{0}] ...' .format(cmd))
        stdout, stderr = p.communicate()
        if p.returncode != 0:
            self.logger.info(stdout)
            self.logger.info(stderr)
            self.logger.error(' Not able to run command: [ {0} ]. See details above.' .format(cmd))
        self.logger.info('         ... Finished!!!' .format(cmd))

        if self.chromSizeInfo is None:
            self.chromSizeInfo = dict()

        chroms = None
        read = False
        counter = 0
        for line in re.split('\n', stdout.decode("utf-8")):
            if re.search('chromCount', line) != None:
                temp = re.split('\s+', line)
                chroms = int(temp[1])
                read = True
                continue

            if read:
                line = line.lstrip().rstrip()
                temp = re.split('\s+', line)

                if temp[0] in self.chromSizeInfo:
                    self.chromSizeInfo[temp[0]] = max( int(temp[2]), self.chromSizeInfo[temp[0]] )
                else:
                    self.chromSizeInfo[temp[0]] = int(temp[2])
                counter += 1

                if counter >= chroms:
                    read = False

    def getBigWigInfo(self):
        """Retrieve chromosome names and their sizes

        BigWigInfo progrom is executed on all listed bigWig files and chromosomes name with respecitve size is stored in :attr:`BigWigHandler.chromSizeInfo` variable.
        From the several listed bigWig files, only largest size of chromosomes are considered.

        """
        for f in self.bigWigFileNames:
            self._getBigWigInfo(f)

        # In case if a single chromosome is required to be extracted
        if self.chromName is not None:
            chromSizeInfo = dict()
            if self.chromName not in self.chromSizeInfo:
                self.logger.error(' The requested chromosome {0} is not found in bigWig files!!.'.format(self.chromName))

            chromSizeInfo[self.chromName] = self.chromSizeInfo[self.chromName]
            self.chromSizeInfo = chromSizeInfo

        # To print chromosome information
        output_line = ''
        chroms = 0
        for chrom in self.chromSizeInfo:
            output_line = output_line + '\n\t\t\t{0}\t\t{1}' .format(chrom, self.chromSizeInfo[chrom])
            chroms += 1
        self.logger.info(' bigWig files {0} contains {1} chromosomes. \n\t\t\tChromosome\tSize {2}' .format(self.bigWigFileNames, chroms, output_line))

    def _bigWigtoWig(self, bigWigFileName, outfilename):
        """Base method to generate Wig file from a bigWig file

        Use :meth:`BigWigHandler.bigWigtoWig` to automatically convert all bigWig files to Wig files.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`BigWigHandler.bigWigtoWig`


        Parameters
        ----------
        bigWigFileName : str
            Input bigWig file names.

        outfilename : str
            Name of output Wig file.

        """
        cmd = None
        if self.chromName is None:
            cmd = '{0} {1} {2}' .format(self.pathTobigWigToWig, bigWigFileName, outfilename)
        else:
            cmd = '{0} -chrom={1} {2} {3}' .format(self.pathTobigWigToWig, self.chromName, bigWigFileName, outfilename)

        args = shlex.split(cmd)
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        self.logger.info(' Started running command: [{0}] ...' .format(cmd))
        stdout, stderr = p.communicate()
        if p.returncode != 0:
            self.logger.info(stdout)
            self.logger.info(stderr)
            self.logger.error(' Not able to convert {0} to bigWig format by command: [ {1} ]. See details above.' .format(self.bigWigFileNames), cmd)
        self.logger.info('         ... Finished!!!' .format(cmd))

    def bigWigtoWig(self, outfilenames=None):
        """ To generate Wig file

        It uses bigWigToWig program to convert bigWig to Wig file.
        Newly generated Wig files name are listed in :attr:`BigWigHandler.WigFileNames`.

        Parameters
        ----------
        outfilenames : str or list of strip
            List of Wig file names. If ``None``, names are automatically generated, files are temporarily created and after execution, all files are deleted.

        """
        if self.WigFileNames is None:
            self.WigFileNames = []

        # Convert to list
        if outfilenames is not None:
            if not isinstance(outfilenames, list):
                if len(self.bigWigFileNames) == 1:
                    self.WigFileNames.append(outfilenames)
                else:
                    dirname = os.path.dirname(outfilenames)
                    basename = os.path.basename(outfilenames)
                    fname, ext = os.path.splitext(basename)
                    for i in range(len(self.bigWigFileNames)):
                        outfile = os.path.join(dirname, '{0}_{1}{2}' .format(fname, i+1, ext))
                        self.WigFileNames.append(outfile)
            else:
                if len(self.bigWigFileNames) != len(outfilenames):
                    raise ValueError(' Number of input bigwig files [{0}] does not match with number of output wig files [{1}].' .format(len(self.bigWigFileNames), len(self.WigFileNames)))
                self.WigFileNames = outfilenames
        else:
            for i in range(len(self.bigWigFileNames)):
                tName = os.path.join(self.workDir, '{0}_{1}.wig.tmp' .format(cmp.name_generator(), i+1) )
                self.WigFileNames.append(tName)
                self.wigToDel = True

        for i in range(len(self.bigWigFileNames)):
            self._bigWigtoWig(self.bigWigFileNames[i], self.WigFileNames[i])

    def saveToH5(self, filename, tmpNumpyArrayFiles=None, title=None, compression='lzf', keep_original=False):
        """Save data to hdf5 file.

        Parameters
        ----------
        filename : str
            Output hdf5 file name with h5 extension.
        tmpNumpyArrayFiles : :class:`TempNumpyArrayFiles` (optional)
             Usually not required. This :class:`TempNumpyArrayFiles` instance stores the temporary numpy array files information.
             To convert large number of bigWig files, its use increases the conversion speed significantly because new temporary array files takes time to generate
             and frequent generation of these files can be avoided.
        title : str (optional)
            Title of the data
        compression : str
            data compression method in HDF5 file : ``lzf`` or ``gzip`` method.
        keep_original : bool
            Whether original data present in bigwig file should be incorporated in HDF5 file. This will significantly increase size of HDF5 file.

        Examples
        --------
        .. code-block:: python

            from hiCMapAnalyze import genomicsDataHandler as gdh

            # start BigWigHandler to combine and convert two bigWig files
            bigwig = gdh.BigWigHandler(['first.bigWig', 'second.bigWig'], './bigWigToWig', './bigWigInfo', maxEntryWrite=1000000)

            # Save hdf5 file
            bigwig.saveToH5('converted.h5')


        """
        if self.chromSizeInfo is None:
            self.logger.info(' No information of chromsomes and their sizes from BigWig File!!! Need to get information...')
            self.getBigWigInfo()

        if self.WigFileNames is None:
            self.logger.info(' No Wig File!!! Need to generate Wig File...')
            self.bigWigtoWig()

        self.wigHandle = WigHandler(self.WigFileNames, self.chromSizeInfo, resolutions=self.resolutions, tmpNumpyArrayFiles=tmpNumpyArrayFiles,
                                     methodToCombine=self.methodToCombine, workDir=self.workDir, maxEntryWrite=self.maxEntryWrite)
        self.wigHandle.convertWigToH5(filename, title=title, compression=compression, keep_original=keep_original)


class WigHandler:
    """To convert Wig files to hdf5 file

    It parses wig files and save all data to a hdf5 file for given resolutions.

    Attributes
    ----------
    WigFileNames : list[str]
        List of Wig files
    chromSizeInfo : dict
        A dictionary containing chromosome size information.
    methodToCombine : str
        method to combine bigWig/Wig files, Presently, accepted keywords are: ``mean``, ``min`` and ``max``
    resolutions : list of str
        List of input resolutions for which data will be stored inside hdf5 file
    tmpNumpyArrayFiles : :class:`TempNumpyArrayFiles`
        This :class:`TempNumpyArrayFiles` instance stores the temporary numpy array files information.
    isWigParsed : bool
        Whether Wig files are already parsed.
    maxEntryWrite : int
        Number of lines read from Wig file at an instant, after this, data is dumped in temporary numpy array file

    Parameters
    ----------
    filenames : str or list(str)
        A Wig file or list of Wig files including path.
    chromSizeInfo : dict
        A dictionary containing chromosome size information. Generated by :meth:`BigWigHandler.getBigWigInfo`.
    resolutions : list[str]
        List of input resolutions for which data will be stored inside hdf5 file
    tmpNumpyArrayFiles : :class:`TempNumpyArrayFiles`
        This :class:`TempNumpyArrayFiles` instance stores the temporary numpy array files information.
    methodToCombine : str
        method to combine bigWig/Wig files, Presently, accepted keywords are: ``mean``, ``min`` and ``max``
    maxEntryWrite : int
        Number of lines read from Wig file at an instant, after this, data is dumped in temporary numpy array file. To reduce memory (RAM) occupancy,
        reduce this number because large numbers need large RAM.


    """
    def __init__(self, filenames, chromSizeInfo=None, resolutions=None, tmpNumpyArrayFiles=None, methodToCombine='mean', workDir=None, maxEntryWrite=10000000):
        self.WigFileNames = filenames

        self.chromSizeInfo = chromSizeInfo                  # dictionary for chromosome sizes, extracted from bigWigInfo program
        self.tmpNumpyArrayFiles = None                        # List of temporary file to store array
        self.isWigParsed  = False

        # Working and output directory
        if workDir is None:
            self.workDir = config['Dirs']['WorkingDirectory']
        else:
            self.workDir = workDir

        self.resolutions = check_resolution_list(resolutions)

        self.logger = logging.getLogger(__name__+'.WigHandler')
        self.logger.setLevel(logging.INFO)

        self.maxEntryWrite = maxEntryWrite # maximum count before writing to output file will be attempted

        # Convert to list
        if not isinstance(filenames, list):
            self.WigFileNames = [ filenames ]

        if not (methodToCombine == 'mean' or methodToCombine == 'max' or methodToCombine == 'min'):
            raise NotImplementedError(' Method [{0}] to combine wig file not implemented. Use: \'mean\', \'max\' or \'min\' ' .format(methodToCombine))
        self.methodToCombine = methodToCombine

        if tmpNumpyArrayFiles is None:
            self.tmpNumpyArrayFiles = TempNumpyArrayFiles(workDir=self.workDir)
            self.tmpNumpyArrayFiles.chromSizeInfo = copy.deepcopy(chromSizeInfo)
        else:
            self._checkExternTempNumpArrayFiles(tmpNumpyArrayFiles)
            self.tmpNumpyArrayFiles = tmpNumpyArrayFiles
            self.tmpNumpyArrayFiles.fillAllArraysWithZeros()

    def _checkExternTempNumpArrayFiles(self, tmpNumpyArrayFiles):
        if not isinstance(tmpNumpyArrayFiles, TempNumpyArrayFiles):
            raise TypeError(' Provided tmpNumpyArrayFiles is not instance of TempNumpyArrayFiles.')

        # check if instance already have information about chromosome size
        if tmpNumpyArrayFiles.chromSizeInfo is None:
            tmpNumpyArrayFiles.chromSizeInfo = copy.deepcopy(self.chromSizeInfo)
        else:
            for key in self.chromSizeInfo:
                tmpNumpyArrayFiles.updateArraysByChromSize(key, self.chromSizeInfo[key])

    def _PerformDataCoarsing(self, Chrom, resolution, coarse_method):
        """Base method to perform Data coarsing.

        This method read temporary Numpy array files and perform data coarsing using the given input method.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._StoreInHdf5File`.

        Parameters
        ----------
        Chrom : str
            Chromosome name

        resolution : str
            resolution in word.

        coarse_method : str
            Name of method to use for data coarsing. Accepted keywords: min, max, median, amean, gmean and hmean.


        """

        output = []
        binsize = cmp.resolutionToBinsize(resolution)
        size = self.chromSizeInfo[Chrom] + 1

        for i in range(1, size, binsize):
            tmpx = None
            if i+binsize >= size:
                tmpx = self.tmpNumpyArrayFiles.arrays[Chrom][i : size]
            else:
                tmpx =  self.tmpNumpyArrayFiles.arrays[Chrom][i : i+binsize]

            int_idx = np.nonzero(tmpx > 0)

            if int_idx[0].shape[0] == 0:
                output.append(0.0)
                continue

            #print(Chrom, tmpx.shape, i, i+binsize, tmpx)
            if coarse_method == 'max':
                output.append(np.amax(tmpx[int_idx]))

            if coarse_method == 'min':
                output.append(np.amin(tmpx[int_idx]))

            if coarse_method == 'amean':
                output.append(np.mean(tmpx[int_idx]))

            if coarse_method == 'hmean':
                output.append(spstats.hmean(tmpx[int_idx]))

            if coarse_method == 'gmean':
                output.append(spstats.gmean(tmpx[int_idx]))

            if coarse_method == 'median':
                output.append(np.median(tmpx[int_idx]))

        # print(Chrom, resolution, coarse_method, size, binsize, size/binsize, len(output), np.amax(output))

        return np.asarray(output)

    def _getChromTitle_parseWig(self, line):
        """To parse chromosome title from the format line of fixedStep and variableStep format Wig file.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        line : str
            The line containing chromosome information from Wig file

        """
        s = re.search('(?<=chrom=)\w+', line)
        if s:
            return s.group(0)

    def _getChromTitleBedgraph_parseWig(self, line):
        """To parse chromosome title from the format line of bedGraph format Wig file.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        line : str
            The line containing chromosome information from Wig file

        """
        s = re.split('\s+', line)
        chrm = re.split(':', s[2])[0]
        return chrm

    def _getStartStepFixedStep_parseWig(self, line):
        """To parse start and step values the format line of fixedStep format Wig file.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        line : str
            The line containing chromosome information from Wig file

        """
        s1 = re.search('(?<=start=)\d+', line)
        if s1:
            start = int(s1.group(0))

        s2 = re.search('(?<=step=)\d+', line)
        if s2:
            step = int(s2.group(0))
        else:
            step = 1

        return int(start), int(step)

    def _getSpan_parseWig(self, line):
        """To parse span value the format line of fixedStep format Wig file.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        line : str
            The line containing chromosome information from Wig file

        """
        s = re.search('(?<=span=)\d+', line)
        if s:
            return int(s.group(0))
        else:
            return 1

    def _FillDataInNumpyArrayFile(self, ChromTitle, location_list, value_list):
        """Fill the extracted data from Wig file to temporary numpy array file

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        ChromTitle : str
            Name of chromosome
        location_list : list of int
            List of locations for given chromosome
        value_list : list of float
            List of values for respecitve chromosome location

        """

        # Write numpy array data to numpy file
        self.tmpNumpyArrayFiles.generateTempNumpyFile(ChromTitle)

        for n in range(len(location_list)):
            if self.methodToCombine == 'mean':
                self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]] = (value_list[n]  + self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]]) / 2
            elif self.methodToCombine == 'max':
                self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]] = max(value_list[n], self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]])
            elif self.methodToCombine == 'min':
                self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]] = min(value_list[n], self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]])
            else:
                self.tmpNumpyArrayFiles.arrays[ChromTitle][location_list[n]] = value_list[n]

        self.tmpNumpyArrayFiles.arrays[ChromTitle].flush()


    def _StoreInHdf5File(self, hdf5Out, title, compression='lzf', keep_original=False):
        """ Base method to perform data coarsing and to store coarsed data in hdf5 file

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler._parseWig`.

        Parameters
        ----------
        hdf5Out : str or :class:`HDF5Handler`
            Name of output hdf5 file or instance of :class:`HDF5Handler`
        title : str
            Title of data
        compression : str
            data compression method in HDF5 file : ``lzf`` or ``gzip`` method.
        keep_original : bool
            Whether original data present in bigwig file should be incorporated in HDF5 file. This will significantly increase size of HDF5 file.

        """
        # In cases when hdf5 filename or hdf5handler object is given as input
        # Initialize new hdf5handler in case file name is given
        if not isinstance(hdf5Out, HDF5Handler):
            hdf5Out = HDF5Handler(hdf5Out, title=title)

        # Writing to hdf5 aur appending to dictionary to return
        self.logger.info(' Writing output file [{0}] ...' .format(hdf5Out.filename))

        for Chrom in self.chromSizeInfo:
            for res in self.resolutions:
                if keep_original:
                    hdf5Out.addDataByArray(Chrom, '1b', 'Original', self.tmpNumpyArrayFiles.arrays[Chrom][:], compression='gzip')
                for coarsing_method in ['min', 'max', 'amean', 'hmean', 'gmean', 'median']:
                    hdf5Out.addDataByArray(Chrom, res, coarsing_method, self._PerformDataCoarsing(Chrom, res, coarsing_method), compression=compression )

        self.logger.info(' \t\t ... Finished writing output file [{0}] ' .format(hdf5Out.filename))
        hdf5Out.close()


    def parseWig(self):
        """ To parse Wig files

        This method parses all Wig files listed in :attr:`WigHandler.WigFileNames`.
        The extracted data is further stored in temporary numpy array files of respective chromosome.
        These numpy array files can be used either for data coarsing or for further analysis.


        * **To save as hdf5:** Use :meth:`WigHandler.convertWigToH5`.
        * **To perform analysis:** Use :meth:`WigHandler.getRawWigDataAsDictionary` to get a dictionary of numpy arrays.

        """
        # Parsing each file
        try:
            for f in self.WigFileNames:
                self._parseWig(f)
        except (SystemExit, KeyboardInterrupt) as e:
            del self.tmpNumpyArrayFiles
            raise e

        self.isWigParsed = True

    def _parseWig(self, wigFileName):
        """ Base method to parse a Wig file.

        This method parses a Wig file and extrated data are copied in temporary numpy array files.

        .. warning::
            **Private method**. Use it at your own risk. It is used internally in :meth:`WigHandler.parseWig`.

        Parameters
        ----------
        wigFileName : str
            Name of Wig File

        """

        # Different format of output wig file
        bedGraph = False
        variableStep = False
        fixedStep = False
        ftype = None

        PreviousChromTitle, ChromTitle, location, value = 'dummy', 'dummy', None, None
        start, step, incr = None, None, None     # for fixedstep type
        span = 1 # default for both fixed and variable step
        dictOut = None # For default return

        fin = open(wigFileName, 'r')

        location_list, value_list = [], []
        ChromList = []
        counter = 0
        for line in fin:

            # To store data in temporary numpy array
            if (ChromTitle != PreviousChromTitle or counter >= self.maxEntryWrite) and not (len(location_list) < 1) :

                self._FillDataInNumpyArrayFile(PreviousChromTitle, location_list, value_list)

                del location_list
                del value_list
                location_list, value_list = [], []
                counter = 0

            # Logging the information
            if PreviousChromTitle != ChromTitle:
                if PreviousChromTitle != 'dummy':
                    self.logger.info('         ... Finished reading and processing for {0} ' .format(PreviousChromTitle))
                self.logger.info(' Identified \'{0}\' format in WIG file; Reading and processing for {1} ... ' .format(ftype, ChromTitle))

            PreviousChromTitle = ChromTitle

            line = line.rstrip().lstrip()

            # Skip blank line
            if not line.strip():
                continue

            if re.search('variableStep', line) is not None:
                variableStep = True
                bedGraph = False
                fixedStep = False
                ChromTitle = self._getChromTitle_parseWig(line)
                ftype = 'variableStep'
                continue

            if re.search('#bedGraph', line) is not None:
                bedGraph = True
                variableStep = False
                fixedStep = False
                ChromTitle = self._getChromTitleBedgraph_parseWig(line)
                ftype = 'bedGraph'
                continue

            if re.search('fixedStep', line) is not None:
                fixedStep = True
                bedGraph = False
                variableStep = False
                ftype = 'fixedStep'
                ChromTitle = self._getChromTitle_parseWig(line)
                start, step = self._getStartStepFixedStep_parseWig(line)
                span = self._getSpan_parseWig(line)
                incr = 0
                continue

            temp = re.split('\s+', line)

            if variableStep:
                location = int(temp[0])
                value = float(temp[1])

            if fixedStep:
                location = start + incr
                value = float(temp[0])
                incr += 1

            if bedGraph:
                value = float(temp[3])

                #Zero based - start from zero - add one

                loc_start = int(temp[1]) + 1
                loc_end = int(temp[2]) + 1

                for l in range(loc_start, loc_end):
                    location_list.append(l)
                    value_list.append(value)
                    counter += 1
            else:
                for g in range(0, span):
                    location = location + g
                    location_list.append(location)
                    value_list.append(value)
                    counter += 1


            location = None
            value = None

        fin.close()

        # Last segment of lines are added in temporary numpy array file
        self._FillDataInNumpyArrayFile(ChromTitle, location_list, value_list)
        self.logger.info('         ... Finished reading and processing for {0} ' .format(ChromTitle))


    def convertWigToH5(self, hdf5Out, title=None, compression='lzf', keep_original=False):
        """To convert Wig files to hdf5 file

        Parameters
        ----------
        hdf5Out : class:`HDF5Handler` or str
            Output hdf5 file name or class:`HDF5Handler` instance
        title : str
            Title of the data
        compression : str
            data compression method in HDF5 file : ``lzf`` or ``gzip`` method.
        keep_original : bool
            Whether original data present in bigwig file should be incorporated in HDF5 file. This will significantly increase size of HDF5 file.

        """
        if not self.isWigParsed:
            self.parseWig()

        # Storing data in hdf5 file
        self._StoreInHdf5File(hdf5Out, title, compression=compression, keep_original=keep_original)

    def getRawWigDataAsDictionary(self, dicOut=None):
        """To get a entire dictionary of data from Wig file

        It generates a dictionary of numpy arrays for each chromosome. These arrays are stored in temporary numpy array files of :class:`TempNumpyArrayFiles`.

        Parameters
        ----------
        dicOut : dict
            The output dictionary to which data will be added or replaced.

        Return
        ------
        dicOut : dict
            The output dictionary.

        """

        noReturn = True
        if dicOut is None:
            dictOut = dict()
            noReturn = False

        if not self.isWigParsed:
            self.parseWig()

        for key in self.chromSizeInfo:
            dictOut[key] = self.tmpNumpyArrayFiles.arrays[key]

        if not noReturn:
            return dicOut


class TextFileHandler:
    """To import a genomic data from column text file format

    It reads text file, make an full array for given shape and fills the missing place with zeros.
    These zeros could be later masked to perform any analysis.

    Example file format:

        ::

            15670000	0.2917373776435852
            15680000	0.2292359322309494
            15690000	0.023434270173311234
            15700000	0.06813383102416992
            15710000	0.13660947978496552
            15720000	0.17478400468826294
            15730000	0.20540907979011536
            .
            .
            .


    This class is used in browser to visualize the genomic data, which are directly imported from text file.

    Attributes
    ----------
    filename : str
        Input text file
    shape : int
        Size of array required to built
    binsize : int
        Size of bins expected in input file. If ``binsize = None``, binsize will be determined from the files, however, it is good to give
        expected binsize to check whether expected binsize match with binsize present in input text file.
    title : str
        Title of the input data
    workDir : str
        Directory where temporary files will be generated. If ``None``, defualt temporary directory of the respective OS will be used.
    data : numpy.ndarray or numpy.memmap
        One-dimensional array containing the data
    tmpNumpyFileName : str
        Name of temporary numpy memory-mapped file

    Parameters
    ----------
    filename : str
        Input text file
    shape : int
        Size of array required to built
    binsize : int
        Size of bins expected in input file. If ``binsize = None``, binsize will be determined from the files, however, it is good to give
        expected binsize to check whther expected binsize match with binsize present in input text file.
    title : str
        Title of the input data
    workDir : str
        Directory where temporary files will be generated. If ``None``, defualt temporary directory of the respective OS will be used.


    """

    def __init__(self, filename, shape, binsize=None, title=None, workDir=None):
        self.filename = filename
        self.shape = shape

        # Working and output directory
        if workDir is None:
            self.workDir = config['Dirs']['WorkingDirectory']
        else:
            self.workDir = workDir

        self.data = None
        self.binsize = None
        self.title = title

        self.tmpNumpyFileName = None

        self.logger = logging.getLogger('TextFileHandler')
        self.logger.setLevel(logging.INFO)

        binsizeInFile = self._getBinSize()
        if binsize is not None:
            if binsizeInFile != binsize:
                raise AssertionError ('Input binsize {0} does not match with the binsize {1} present in file.'.format(binsize, binsizeInFile))
        self.binsize = binsizeInFile

    def __del__(self):
        self._removeTempNumpyFile()

    def _removeTempNumpyFile(self):
        """Remove temporary numpy memory-mapped file
        """
        if self.tmpNumpyFileName is not None:
            del self.data
            try:
                os.remove(self.tmpNumpyFileName)
            except:
                pass

    def _generateTempNumpyFile(self):
        """Generate temporary numpy memory-mapped file
        """
        (fd, fname) = tempfile.mkstemp(suffix='.npy', prefix='np_', dir=self.workDir, text=False)
        os.close(fd)     # Close file, error in windows OS
        self.logger.info(' Generating temporary numpy array file [{0}] ...' .format(fname))

        size = self.shape + 1        # Be careful: added one to easiliy handle real locations, zeroth index is dummy, dont use zeroth location
        np.save(fname, np.zeros(size, dtype=np.float))
        self.data = np.load(fname, mmap_mode='r+')
        self.tmpNumpyFileName = fname

    def _getBinSize(self):
        """Determine binsize from input file
        """
        fin = open(self.filename, 'r')

        listBinSize = []
        count = 0
        prevLocation = None
        for line in fin:
            line = line.lstrip().rstrip()

            if not line.strip():    continue

            temp = re.split('\s+', line)

            newLocation = int(temp[0])
            if prevLocation is not None:
                listBinSize.append( abs(newLocation - prevLocation) )
            prevLocation = newLocation

            if count > 100:
                break

            count = count + 1

        fin.close()

        return np.amin( listBinSize )

    def readData(self):
        """Read data from input file

        Read data from input file and store in :attr:`TextFileHandler.data` as one-dimensional array
        """
        if self.shape > 100000:
            self._generateTempNumpyFile()
        else:
            size = self.shape + 1
            self.data = np.zeros(size, dtype=np.float)

        fin = open(self.filename, 'r')

        try:
            for line in fin:
                line = line.lstrip().rstrip()

                if not line.strip():    continue

                temp = re.split('\s+', line)

                idx = int( int(temp[0])/self.binsize )

                try:
                    self.data[idx] = float(temp[1])
                except IndexError:
                    break
                    pass

        except (SystemExit, KeyboardInterrupt) as e:
            self._removeTempNumpyFile()
            raise e

        fin.close()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import re, os
import logging
import tempfile
import numpy as np
import tarfile
import zipfile

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import ccmap as cmp
from . import ccmapHelpers as cmh
from . import gcmap as gmp

class CooMatrixHandler:
    """To import ccmap from files similar to sparse matrix in Coordinate (COO) format

    Two types of coordinates are accepted:
        * with ``coordinate='real'`` pair of absolute binned locations on chromosome
        * with ``coordinate='index'`` row and column index of matrix. index **should** always start from zero for absoulte beginning of chromosome.
          e.g. for 10kb, 0-10000 should have index of zero, 10000-20000 have index of one. If this is file format, resolution should be provided explicitly.

    .. warning::
        Input file should contain matrix for **only one** chromosome.

    Following file format can be read as a text file, where first and second column is location on chromosome and third column is the value:
    ::

        20000000	20000000	2692.0
        20000000	20100000	885.0
        20100000	20100000	6493.0
        20000000	20200000	15.0
        20100000	20200000	52.0
        20200000	20200000	2.0
        20000000	20300000	18.0
        20100000	20300000	40.0
        .
        .
        .
        .
        .
        .


    To Instantiate this class, three scenarios are possible:
        * ``Text in Archive``: Both a list of input files and a compressed file is given. It means look for input files in compressed file.
        * ``Text``: Only a list of input files is given. It means, read data directly from input file.
        * ``Archive``: Only a compressed file is given. It means, read all files present in compressed files.

    Parameters
    ----------
    inputFiles : str or list
        name of a input file or list of input files
    inputCompressedFile : str
        name of input tar archive
    mapType : str
    	Type of HiC map

    		* ``intra``: for intra-chromosomal map
    		* ``inter``: for inter-chromosomal map
    resolution : str
        resolution of HiC map. Example: '100kb', '10kb' or '25kb' etc. If ``None``, resolution will be determined from the input file.
    workDir : str
        Directory where temporary files will be stored.


    Attributes
    ----------
    inputFileList : list
        List of input files. It could be ``None`` when not provided.
    inputType : str
        Type of input. Three types: ``Text``, ``Text in Archive`` and ``Archive`` are determined from user input.
    inputCompressedFile : str
        Name of input compressed file. It could be ``None`` when not provided.
    outputFileList : str
        List of Output files
    compressType : str
        Format of compressed file. It could be etiher ``.tar`` or ``.zip`` or ``None``
    compressHandle : ZipFile or TarFile
        An object to handle compressed file. It could be `ZipFile <https://docs.python.org/3/library/zipfile.html#zipfile-objects>`_ or
        `TarFile <https://docs.python.org/3/library/tarfile.html#tarfile-objects>`_ instance depending on compressed format.
    mapType : str
    	Types of HiC map
    	* ``intra``: for intra-chromosomal map
    	* ``inter``: for inter-chromosomal map
    coordinate : str
        Coordinate type in input text file. It could be either ``real`` for real locations or ``index`` for rows and column indices.
    resolution : str
        resolution of HiC map. Example: '100kb', '10kb' or '25kb' etc. If ``None``, resolution will be determined from the input file.

        If ``coordinate='index'``, resolution is essential for further processing.
    workDir : str
        Directory where temporary files will be stored. If not provided, directory name will be taken from configuration file.

    """

    def __init__(self, inputFiles=None, inputCompressedFile=None, mapType='intra', resolution=None, coordinate='real', workDir=None, logHandler=None):

        # Checking both text and compressed input files
        self.inputType = self._checkInputType(inputFiles, inputCompressedFile)

        # Initial processing for archive file
        self.inputCompressedFile = inputCompressedFile
        self.compressType = self._checkCompressedFile()

        # Make a input file list if user has provided one
        self.inputFileList = None
        if inputFiles is not None:
            self.inputFileList = []
            self._addInputTextFiles(inputFiles)

        # If user has not provided input files, read file list from compressed file and return handle object of compressed file
        self.compressHandle = None
        self._extractCompressedFile()

        # Working and output directory
        if workDir is not None:
            self.workDir = workDir
        else:
            self.workDir = config['Dirs']['WorkingDirectory']

        # map type
        self.mapType = mapType

        # Checking corrdinate type: index or real location
        if coordinate == 'real':
            pass
        elif coordinate == 'index':
            if resolution is None:
                raise ValueError ("Please provide resolution because coordinate values are indexes of rows and columns")
        else:
            raise ValueError (" Keyword [{0}] is not allowed. Use 'real' or 'index'." .format(coordinate))
        self.resolution = resolution
        self.coordinate = coordinate

        # Output file list
        self.outputFileList = None

        # Labels
        self.xlabels = None
        self.ylabels = None

        # logger
        self.logHandler = logHandler
        self.logger = logging.getLogger('CooMatrixHandler')
        if logHandler is not None:
            self.logger.propagate = False
            self.logger.addHandler(logHandler)
        self.logger.setLevel(logging.INFO)

    def __del__(self):
        if self.logHandler is not None:
            self.logger.removeHandler( self.logHandler )

    def _addInputTextFiles(self, inputFiles):
        """ Add Input Files

        It add input files and also convert to list

        Parameters
        ----------
        inputFiles : str or list
            name of a input file or list of input files

        """
        if isinstance(inputFiles, list):
            self.inputFileList = self.inputFileList + inputFiles
        else:
            self.inputFileList.append(inputFiles)

    def _checkInputType(self, inputFiles, compressedFile):
        """To check input type given by user

        There are three possible scenarios:
            * ``Text in Archive``: Both a list of input files and a compressed file is given. It means look for input files in compressed file.
            * ``Text``: Only a list of input files is given. It means, read data directly from input file.
            * ``Archive``: Only a compressed file is given. It means, read all files present in compressed files.

        Parameters
        ----------
        inputFiles : str or list
            name of a input file or list of input files.
        compressedFile : str
            Name of compressed file

        Raises
        ------
        ValueError
            If both input files and compressedFile is ``None``

        """
        inputType = None
        # Three porabable type of inputs 'Text' 'Text in Archive' 'Archive'
        if inputFiles is not None and compressedFile is not None:
            inputType = 'Text in Archive'
        elif inputFiles is not None and compressedFile is None:
            inputType = 'Text'
        elif inputFiles is None and compressedFile is not None:
            inputType = 'Archive'
        else:
            raise ValueError ("Both inputFiles and inputCompressedFile cannot be None.")

        return inputType

    def _checkCompressedFile(self):
        """Determine format of compressed file.

        Returns
        -------
        compressType : str
            One of the two possible keywords (``tar`` and ``zip``) or ``None``.

        Raises
        ------
        NotImplementedError
            If format is not ``zip``, ``tar.gz`` or ``tar.bz2``.

        """
        if self.inputCompressedFile is None:  return
        compressType = None
        file_extension = os.path.splitext(self.inputCompressedFile)[1]
        if file_extension == '.zip':
            compressType = 'zip'
        elif file_extension == '.gz' or file_extension == '.tar' or file_extension == '.bz2':
            compressType = 'tar'
        else:
            raise NotImplementedError (" [{0}] compression format is not implemented. Please use '.zip' or '.tar.gz' or '.tar.bz2' compression format." .format(file_extension))

        return compressType

    def _extractCompressedFile(self):
        """Generate input file list from compressed file
        """
        if self.inputCompressedFile is None:    return

        if self.compressType == 'zip':
            self.compressHandle = zipfile.ZipFile(self.inputCompressedFile, 'r')
            if self.inputFileList is None:
                self.inputFileList = self.compressHandle.namelist()
        else:
            self.compressHandle = tarfile.open(self.inputCompressedFile, 'r')
            if self.inputFileList is None:
                self.inputFileList = self.compressHandle.getnames()

    def _read_a_map_text_file(self, filename):
        """ Generate CCMAP object from a text file with column format --- i  j  value

        To generate new CCMAP oject from a input text file. The file should contain three column where first and second column indicate
        location on the chromosome and third column indicate the contact frequency

        Parameters
        ----------
        filename : str
        	Name of input text file.

        Returns
        -------
        ccMapObj : :class:`hiCMapAnalyze.HiCMapMain.CCMAP`
        	A CCMAP object

        """

        fin = open(filename, 'r')

        i, j, value = [], [], []

        print('  ')
        self.logger.info(' Reading file: [{0}]... ' .format(filename))

        binsize = None
        if self.coordinate != 'real':
            binsize = cmp.resolutionToBinsize(self.resolution)

        for line in fin:
            line = line.rstrip('\n')
            if not line.strip():
                continue
            temp = line.split()
            if self.coordinate == 'real':
                i.append(int(temp[0]))
                j.append(int(temp[1]))
            else:
                i.append(int(temp[0])*binsize)
                j.append(int(temp[1])*binsize)

            value.append(float(temp[2]))

        self.logger.info('     ... Finished reading file: [{0}] ' .format(filename))

        ccMapObj = gen_map_from_locations_value(i, j, value, resolution=self.resolution, workDir=self.workDir, mapType=self.mapType)
        self.resolution = cmp.binsizeToResolution(ccMapObj.binsize)

        del i
        del j
        del value

        return ccMapObj

    def _read_map_from_compressed_file(self, mapfile):
        """ Generate CCMAP object from a file, which is inside a tar archive

        If HiC map file is inside a tar archive, this method generate CCMAP object by directly reading the map file without extracting the tar archive.

        Parameters
        ----------
        mapfile : str
        	Name of input map file with full relative path inside the tar archive

        Returns
        -------
        ccMapObj : ``None`` or :class:`hiCMapAnalyze.HiCMapMain.CCMAP`
        	A CCMAP object is retured. However, if input file is not found inside tar archive, ``None`` is returned.

        """

        # Opening tar file
        # tar = tarfile.open(self.inputCompressedFile, "r")

        print('  ')
        self.logger.info(' Extracting-Reading [{0}] from [{1}]... ' .format(mapfile, self.inputCompressedFile))

        try:
            # extract the specific file to file stream
            if self.compressType == 'tar':
                fin = self.compressHandle.extractfile(mapfile)
            else:
                fin = self.compressHandle.open(mapfile)
        except KeyError:
            self.logger.warning(" [{0}] not found in [{1}]... \n\t\t...Skipping this file..." .format(mapfile, self.inputCompressedFile))
            return


        i, j, value = [], [], []

        binsize = None
        if self.coordinate != 'real':
            binsize = cmp.resolutionToBinsize(self.resolution)

        while (1):

            line = fin.readline().decode()

            if not line :
            	break

            line = line.rstrip('\n')

            if not line.strip():
            	continue

            temp = line.split()
            if self.coordinate == 'real':
                i.append(int(temp[0]))
                j.append(int(temp[1]))
            else:
                i.append(int(temp[0])*binsize)
                j.append(int(temp[1])*binsize)
            value.append(float(temp[2]))

        self.logger.info('   ...Finished extracting and reading [{0}]' .format(mapfile))

        ccMapObj = gen_map_from_locations_value(i, j, value, resolution=self.resolution, workDir=self.workDir, mapType=self.mapType)
        self.resolution = cmp.binsizeToResolution(ccMapObj.binsize)

        del i
        del j
        del value

        return ccMapObj

    def setOutputFileList(self, outputFiles):
        """To set list of output files

        Parameters
        ----------
        outputFiles : str or list
            Name of a output file or list of output files. For each input file,
            a respective output file will be generated, therefore, number of
            input and output files should match.

        """
        if not isinstance(outputFiles, list):
            self.outputFileList = [ outputFiles ]
        else:
            self.outputFileList = outputFiles

        if len(self.outputFileList) != len(self.inputFileList):
            raise AssertionError (" Number of input files [{0}] does not match"
                    + "with output files [{1}]. " .format(
                        len(self.inputFileList), len(self.outputFileList)) )

    def setLabels(self, xlabels, ylabels):
        """To set xlabels and ylabels for contact maps

        xlabel and ylabel act as a title of data along X-axis
        and Y-axis respectively.

        Parameters
        ----------
        xlabels : str or list
            Name of the data along X-axis or list of names of the data for respective input files.
        ylabels : str or list
            Name of the data along y-axis or list of names of the data for respective input files.

        """
        if not isinstance(xlabels, list):
            self.xlabels = [ xlabels ]
        else:
            self.xlabels = xlabels

        if not isinstance(xlabels, list):
            self.ylabels = [ ylabels ]
        else:
            self.ylabels = ylabels

        if len(self.xlabels) != len(self.inputFileList):
            raise AssertionError (" Number of input files [{0}] does not match"
                                    + "with xlabels [{1}]. " .format(
                                    len(self.inputFileList),
                                    len(self.xlabels)) )

        if len(self.ylabels) != len(self.inputFileList):
            raise AssertionError (" Number of input files [{0}] does not match"
                                    + "with ylabels [{1}]. " .format(
                                    len(self.inputFileList),
                                    len(self.ylabels)) )

    def save_ccmaps(self, outputFiles=None, xlabels=None, ylabels=None,
                    compress=True):
        """To Save all Hi-C maps

        This function reads input files one by one and save it as a ``.ccmap``
        file.

        Parameters
        ----------
        outputFiles : str or list
            Name of a output file or list of output files. For each input file,
            a respective output file will be generated, therefore, number of
            input and output files should match.
        xlabels : str or list
            Name of the data along X-axis or list of names of the data for
            respective input files.
        ylabels : str or list
            Name of the data along y-axis or list of names of the data for
            respective input files. if it is ``None``, ylabels will be same as
            xlabels.
        compress : bool
        	If ``True``, numpy array (matrix) file will be compressed to reduce
            storage memory.

        """

        if outputFiles is not None:
            self.setOutputFileList(outputFiles)

        if outputFiles is None and self.outputFileList is None:
            raise ValueError('Output file names not provided.')

        # xlabels and ylabels handling
        if xlabels is not None and ylabels is not None:
            self.setLabels(xlabels, ylabels)

        if xlabels is not None and ylabels is None:
            self.setLabels(xlabels, xlabels)

        if xlabels is None and self.xlabels is None:
            raise ValueError('xlabels not provided.')

        for i in range(len(self.inputFileList)):
            ccmap = None

            try:
                if self.inputType == 'Text in Archive' \
                        or self.inputType == 'Archive':
                    ccmap = self._read_map_from_compressed_file(
                                    self.inputFileList[i])
                else:
                    ccmap = self._read_a_map_text_file(self.inputFileList[i])

                if ccmap is not None:
                    ccmap.xlabel = self.xlabels[i]
                    ccmap.ylabel = self.ylabels[i]

                    cmp.save_ccmap(ccmap, self.outputFileList[i],
                                    compress=compress)
                    del ccmap
                else:
                    self.logger.warning("   Not able to read [{0}] ... "
                                + "Skipped!!" .format(self.inputFileList[i]))

            except (KeyboardInterrupt, SystemExit) as e:
                if ccmap is not None:
                    del ccmap
                raise e

    def save_gcmap(self, outputFile, xlabels=None, ylabels=None,
                    coarsingMethod='sum', compression='lzf'):
        """To Save all Hi-C maps as a gcmap file

        This function reads input files one by one and save it as a ``.gcmap``
        file.

        Parameters
        ----------
        outputFile : str
            Name of a output gcmap file.
        xlabels : str or list
            Name of the data along X-axis or list of names of the data for
            respective input files.
        ylabels : str or list
            Name of the data along y-axis or list of names of the data for
            respective input files. if it is ``None``, ylabels will be same as
            xlabels.
        coarsingMethod : str
            Method of downsampling. Three accepted methods are ``sum``: sum all
            values, ``mean``: Average of all values and ``max``: Maximum of
            all values.
        compression : str
            Compression method. Presently allowed : ``lzf`` for LZF compression
            and ``gzip`` for GZIP compression.

        """

        # xlabels and ylabels handling
        if xlabels is not None and ylabels is not None:
            self.setLabels(xlabels, ylabels)

        if xlabels is not None and ylabels is None:
            self.setLabels(xlabels, xlabels)

        if xlabels is None and self.xlabels is None:
            raise ValueError('xlabels are not provided.')

        for i in range(len(self.inputFileList)):
            ccmap = None

            try:
                if self.inputType == 'Text in Archive' \
                        or self.inputType == 'Archive':
                    ccmap = self._read_map_from_compressed_file(
                                    self.inputFileList[i])
                else:
                    ccmap = self._read_a_map_text_file(self.inputFileList[i])

                if ccmap is not None:
                    ccmap.xlabel = self.xlabels[i]
                    ccmap.ylabel = self.ylabels[i]
                    gmp.addCCMap2GCMap(ccmap, outputFile,
                                       compression = compression,
                                       generateCoarse = True,
                                       coarsingMethod = coarsingMethod)

                    del ccmap
                else:
                    self.logger.warning("   Not able to read [{0}] ... "
                                + "Skipped!!" .format(self.inputFileList[i]))

            except (KeyboardInterrupt, SystemExit) as e:
                if ccmap is not None:
                    del ccmap
                raise e

class PairCooMatrixHandler:
    """To import ccmap from files similar to paired sparse matrix Coordinate (COO) format

    This format is very similar to COO format with addiotional infromation of chromosome. Therefore,
    maps for all chromosome could be contained in a single file.

    This type of format appeared with following publication:
        * http://dx.doi.org/10.1016/j.cell.2015.10.026 --- https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE72512

    Following file format can be read as a text file, where first and second column is location on chromosome and third column is the value:
    ::

        chr4     60000   75000   chr4    60000   75000   0.1163470887070292
        chr4     60000   75000   chr4    105000  120000  0.01292745430078102
        chr4     60000   75000   chr4    435000  450000  0.01292745430078102
        chr4     75000   90000   chr4    75000   90000   0.05170981720312409
        chr4     75000   90000   chr4    345000  360000  0.01292745430078102
        chr4     90000   105000  chr4    90000   105000  0.01292745430078102
        .
        .
        .
        .
        .
        .


    Parameters
    ----------
    inputFile : str
        name of a input file
    ccmapOutDir : str
        name of directory where all ccmap file will be stored.
    ccmapSuffix : str
    	Suffix for ccmap file name.
    gcmapOut : str
        Name of output gcmap file.
    workDir : str
        Directory where temporary files will be stored.


    Attributes
    ----------
    inputFile : str
        name of a input file
    ccmapOutDir : str
        name of directory where all ccmap file will be stored.
    ccmapSuffix : str
    	Suffix for ccmap file name.
    gcmapOut : str
        Name of output gcmap file.
    gcmapOutOptions : dict
        Dictionary for gcmap output options.
    workDir : str
        Directory where temporary files will be stored.


    Examples
    --------
        >>> pair_map_handle = PairCooMatrixHandler('GSM1863750_tethered_rep1_contacts.txt', gcmapOut='GSM1863750_tethered_rep1_contacts.gcmap')
        >>> pair_map_handle.setGCMapOptions()
        >>> pair_map_handle.runConversion()


    """

    def __init__(self, inputFile, ccmapOutDir=None, ccmapSuffix=None, gcmapOut=None, workDir=None, logHandler=None):
        self.inputFile = inputFile
        self.ccmapOutDir = ccmapOutDir
        self.ccmapSuffix = ccmapSuffix
        self.gcmapOut = gcmapOut
        self.gcmapOutOptions = None
        self._fin = None
        self.workDir = workDir
        self.logHandler = logHandler

        # Working and output directory
        if workDir is not None:
            self.workDir = workDir
        else:
            self.workDir = config['Dirs']['WorkingDirectory']
        # logger
        self.logHandler = logHandler
        self.logger = logging.getLogger('PairCooMatrixHandler')
        if logHandler is not None:
            self.logger.propagate = False
            self.logger.addHandler(logHandler)
        self.logger.setLevel(logging.INFO)

    def setGCMapOptions(self, compression='lzf', generateCoarse=True, coarsingMethod='sum', replaceCMap=True):
        """ Set options for output gcmap file

        Parameters
        ----------
        compression : str
            Compression method. Presently allowed : ``lzf`` for LZF compression and ``gzip`` for GZIP compression.
        generateCoarse : bool
            Also generates all coarser maps where resolutions will be coarsed by a factor of two, consequetively.
            e.g.: In case of 10 kb input resolution, downsampled maps of ``20kb``, ``40kb``, ``80kb``, ``160kb``, ``320kb`` etc.
            will be generated untill, map size is less than 500.
        coarsingMethod : str
            Method of downsampling. Three accepted methods are ``sum``: sum all values, ``mean``: Average of all values
            and ``max``: Maximum of all values.
        replaceCMap : bool
            Replace entire old ccmap data including resolutions and coarsed data.

        """
        self.gcmapOutOptions = dict()
        self.gcmapOutOptions['compression'] = compression
        self.gcmapOutOptions['generateCoarse'] = generateCoarse
        self.gcmapOutOptions['coarsingMethod'] = coarsingMethod
        self.gcmapOutOptions['replaceCMap'] = replaceCMap

    def _convertProcessedValues(self, i, j, value, chrom):
        self.logger.info('##### CONVERTING FOR {0} #######'.format(chrom))
        ccmap = gen_map_from_locations_value(i, j, value, mapType='intra', workDir=self.workDir, logHandler=self.logHandler)
        ccmap.xlabel = chrom
        ccmap.ylabel = chrom

        if self.ccmapOutDir is not None or self.ccmapSuffix is not None:
            self._save_ccmap(ccmap, chrom, cmp.binsizeToResolution(ccmap.binsize))

        if self.gcmapOut is not None:
            self._save_gcmap(ccmap)

        del ccmap
        self.logger.info('##### ################# #######\n'.format(chrom))

    def _save_ccmap(self, ccmap, title, resolution):
        if self.ccmapOutDir is None:
            self.ccmapOutDir = os.getcwd()

        if self.ccmapSuffix is None:
            outFileName = title + '_' + resolution + '.ccmap'
        else:
            outFileName = title + '_' + resolution + '_' + self.ccmapSuffix + '.ccmap'

        fullOutPath = os.path.join(self.ccmapOutDir, outFileName)
        cmp.save_ccmap(ccmap, fullOutPath, compress=True)


    def _save_gcmap(self, ccmap):
        if self.gcmapOutOptions is None:
            raise ValueError('No options set for output gcmap file!!. Use PairCooMatrix.setGCMapOptions() to set the options.')

        compression = self.gcmapOutOptions['compression']
        generateCoarse = self.gcmapOutOptions['generateCoarse']
        coarsingMethod = self.gcmapOutOptions['coarsingMethod']
        replaceCMap = self.gcmapOutOptions['replaceCMap']
        gmp.addCCMap2GCMap(ccmap, self.gcmapOut,
                                compression=compression,
                                generateCoarse=generateCoarse,
                                coarsingMethod=coarsingMethod,
                                replaceCMap=replaceCMap,
                                logHandler=self.logHandler)

    def runConversion(self):
        """ Perform conversion and save to ccmap and/or gcmap file.

        Read the input file, process the data, and convert it to ccmap
        or gcmap file. For output gcmap, :meth:`PairCooMatrixHandler.setGCMapOptions`
        should be called to set the neccessary options.
        """
        # Check if gcmap output is given and gcmap options are set
        if self.gcmapOutOptions is None and self.gcmapOut is not None:
            raise ValueError('No options set for output gcmap file!!. Use PairCooMatrix.setGCMapOptions() to set the options.')

        # At the start of file
        if self._fin is None:
            self._fin = open(self.inputFile, 'r')
        else:
            self._fin.seek(0)

        prevChrom = 'dummy'
        i, j, value = [], [], []
        for line in self._fin:

            # Process and break line
            line = line.rstrip().lstrip()
            temp = re.split('\s+', line)

            # Check for intra-chromosomal contact map
            if temp[0] == temp[3] and not temp[0] == prevChrom and value:
                self._convertProcessedValues(i, j, value, prevChrom)
                self.logger.info('##### Started Reading For {0} #######'.format(temp[0]))

                i, j, value = [], [], []

            # assign chromosome name
            prevChrom = temp[0]

            # Append only in case of intra-chromosomal map
            if temp[0] == temp[3]:
                i.append(int(temp[2]))
                j.append(int(temp[5]))
                value.append(float(temp[6]))

        # Convert for last map --- intra-chromosomal
        if temp[0] == temp[3]:
            self._convertProcessedValues(i, j, value, temp[0])
            i, j, value = [], [], []

class HomerInputHandler:
    """To import ccmap from Hi-C maps generated by HOMER

    HOMER package generates the `Hi-C interaction matrices in text file <http://homer.salk.edu/homer/interactions/HiCmatrices.html>`_. This Hi-C interaction file can be imported using this class.
    HOMER format interaction matrix file may contain data for all the chromsomes while this class separatly read and save matrix of each chromosome.

    To Instantiate this class, three scenarios are possible:
        * ``Text in Archive``: Both a list of input files and a compressed file is given. It means look for input files in compressed file.
        * ``Text``: Only a list of input files is given. It means, read data directly from input file.
        * ``Archive``: Only a compressed file is given. It means, read all files present in compressed files.

    Parameters
    ----------
    inputFiles : str or list
        Name of a input file or list of input files. If ``None``, all files from compressed files are used as input files.
    inputCompressedFile : str
        Name of input compressed file. Accepted formats: ``tar.gz``, ``tar.bz2`` and ``zip``.
    workDir : str
        Directory where temporary files will be stored. If it is not provided, this value is taken from configuration file.

    Attributes
    ----------
    inputFileList : list
        List of input files. It could be ``None`` when not provided.
    inputType : str
        Type of input. Three types: ``Text``, ``Text in Archive`` and ``Archive`` are determined from user input.
    inputCompressedFile : str
        Name of input compressed file. It could be ``None`` when not provided.
    compressType : str
        Format of compressed file. It could be etiher ``.tar`` or ``.zip`` or ``None``
    compressHandle : ZipFile or TarFile
        An object to handle compressed file. It could be `ZipFile <https://docs.python.org/3/library/zipfile.html#zipfile-objects>`_ or
        `TarFile <https://docs.python.org/3/library/tarfile.html#tarfile-objects>`_ instance depending on compressed format.
    workDir : str
        Directory where temporary files will be stored.
    fIns : list[output file stream]
        Input file stream for each input files
    chromList : list[str]
        List of chromosome found in input files
    resolution : str
        Resolution of map
    fTmpOutNames : list[str]
        List of temporary output files where data for each chromosomes are extracted separately
    fTmpOut : list[output file stream]
        List of output file streams for respective temporary output files

    """

    def __init__(self, inputFiles=None, inputCompressedFile=None, workDir=None, logHandler=None):

        # Checking both text and compressed input files
        self.inputType = self._checkInputType(inputFiles, inputCompressedFile)  # raise error when both inputs are None

        # Initial processing for archive file
        self.inputCompressedFile = inputCompressedFile
        self.compressType = self._checkCompressedFile()

        # Make a input file list if user has provided one
        self.inputFileList = None
        if inputFiles is not None:
            self.inputFileList = []
            self._addInputTextFiles(inputFiles)

        # If user has not provided input files, read file list from compressed file and return handle object of compressed file
        self.compressHandle = None
        self._extractCompressedFile()

        # Set working directory
        if workDir is not None:
            self.workDir = workDir
        else:
            self.workDir = config['Dirs']['WorkingDirectory']

        self.chromList = None
        self.resolution = None
        self.headers = None
        self.fTmpOut = None
        self.fTmpOutNames = None
        self.fIns = None

        # logger
        # logger
        self.logHandler = logHandler
        self.logger = logging.getLogger('HomerInputHandler')
        if logHandler is not None:
            self.logger.propagate = False
            self.logger.addHandler(logHandler)
        self.logger.setLevel(logging.INFO)

    def __del__(self):
        self._removeTemporaryOutputFiles()
        if self.logHandler is not None:
            self.logger.removeHandler( self.logHandler )

    def _checkInputType(self, inputFiles, compressedFile):
        """To check input type given by user

        There are three possible scenarios:
            * ``Text in Archive``: Both a list of input files and a compressed file is given. It means look for input files in compressed file.
            * ``Text``: Only a list of input files is given. It means, read data directly from input file.
            * ``Archive``: Only a compressed file is given. It means, read all files present in compressed files.

        Parameters
        ----------
        inputFiles : str or list
            name of a input file or list of input files.
        compressedFile : str
            Name of compressed file

        Raises
        ------
        ValueError
            If both input files and compressedFile is ``None``

        """
        inputType = None
        # Three porabable type of inputs 'Text' 'Text in Archive' 'Archive'
        if inputFiles is not None and compressedFile is not None:
            inputType = 'Text in Archive'
        elif inputFiles is not None and compressedFile is None:
            inputType = 'Text'
        elif inputFiles is None and compressedFile is not None:
            inputType = 'Archive'
        else:
            raise ValueError ("Both inputFiles and inputCompressedFile cannot be None.")

        return inputType

    def _checkCompressedFile(self):
        """Determine format of compressed file.

        Returns
        -------
        compressType : str
            One of the two possible keywords (``.tar`` and ``.zip``) or ``None``.

        Raises
        ------
        NotImplementedError
            If format is not ``zip``, ``tar.gz`` or ``tar.bz2``.

        """
        if self.inputCompressedFile is None:  return
        compressType = None
        file_extension = os.path.splitext(self.inputCompressedFile)[1]
        if file_extension == '.zip':
            compressType = 'zip'
        elif file_extension == '.gz' or file_extension == '.tar' or file_extension == '.bz2':
            compressType = 'tar'
        else:
            raise NotImplementedError (" [{0}] compression format is not implemented. Please use '.zip' or '.tar.gz' or '.tar.bz2' compression format." .format(file_extension))

        return compressType

    def _addInputTextFiles(self, inputFiles):
        """ Add Input Files

        It add input files and also convert to list

        Parameters
        ----------
        inputFiles : str or list
            name of a input file or list of input files

        """
        if isinstance(inputFiles, list):
            self.inputFileList = self.inputFileList + inputFiles
        else:
            self.inputFileList.append(inputFiles)

    def _extractCompressedFile(self):
        """Generate input file list from compressed file
        """
        if self.inputCompressedFile is None:    return

        if self.compressType == 'zip':
            self.compressHandle = zipfile.ZipFile(self.inputCompressedFile, 'r')
            if self.inputFileList is None:
                self.inputFileList = self.compressHandle.namelist()
        else:
            self.compressHandle = tarfile.open(self.inputCompressedFile, 'r')
            if self.inputFileList is None:
                self.inputFileList = self.compressHandle.getnames()

    def _openInputFiles(self):
        """Open input files
        Open all input files and store input file stream in :attr:`HOMERInputHandler.fIns` list

        """
        if self.fIns is None:
            self.fIns = []
        else:
            self._closeInputFiles()
            self.fIns = []

        for f in self.inputFileList:
            # First text file
            if self.inputType == 'Text':
                fin = open(f, 'r')
                self.fIns.append(fin)
            else:
                fin = None
                # Zip file
                if self.compressType == 'zip':
                    try:
                        fin = self.compressHandle.open(f)
                    except KeyError:
                        self.logger.warning(' {0} not found in [{1}]... \n\t\t...Skipping this file...'.format(f, self.inputCompressedFile))
                else:
                    # Tar file
                    try:
                        fin = self.compressHandle.extractfile(f)
                    except KeyError:
                        self.logger.warning(' [{0}] not found in [{1}]... \n\t\t...Skipping this file...'.format(f, self.inputCompressedFile))
                self.fIns.append(fin)

    def _closeInputFiles(self):
        """Close all input files
        Close all input file stream present in :attr:`HOMERInputHandler.fIns` list
        """
        if self.fIns is not None:
            for fin in self.fIns:
                if fin is not None:
                    # close function is not available when tar file object is used
                    if self.compressType != 'tar':
                        fin.close()
            del self.fIns
            self.fIns = None

    def _generateTemporaryOutputFiles(self):
        """Generate temporary text files

        Generate temporary text file for each chromosome and matrix is stored as --- i j value --- format.

        """
        if self.fTmpOut is None:
            self.fTmpOut = dict()
        else:
            self._closeTemporaryOutputFiles()
            self.fTmpOut = dict()

        if self.fTmpOutNames is None:
            self.fTmpOutNames = dict()
        else:
            self._removeTemporaryOutputFiles()
            self.fTmpOutNames = dict()

        self._getChromListAndResolution()

        for chrom in self.chromList:

            # Only filename is generated
            fd, fname = tempfile.mkstemp(prefix='{0}_' .format(chrom), suffix='.tmp', dir=self.workDir)
            os.close(fd)
            try:
                os.remove(fname)
            except:
                pass

            fout = open(fname, 'w')
            self.fTmpOut[chrom] = fout
            self.fTmpOutNames[chrom] = fname

    def _closeTemporaryOutputFiles(self):
        """Close all temporary files
        Close all output file stream present in :attr:`HOMERInputHandler.fTmpOut` list
        """
        if self.fTmpOut is not None:
            for key in self.fTmpOut:
                self.fTmpOut[key].close()
            self.fTmpOut = None

    def _openTemporaryOutputFiles(self):
        """Open all temporary files
        Open all output files present in :attr:`HOMERInputHandler.fTmpOutNames` list
        """
        if self.fTmpOut is not None:
            for key in self.fTmpOut:
                self.fTmpOut[key].close()
            self.fTmpOut = None

        self.fTmpOut = dict()
        for key in self.fTmpOutNames:
            self.fTmpOut[key] = open(self.fTmpOutNames[key], 'r')

    def _removeTemporaryOutputFiles(self):
        """Remove all temporary files
        Delete all temporary files present in :attr:`HOMERInputHandler.fTmpOutNames` list from external storage
        """
        self._closeTemporaryOutputFiles()
        if self.fTmpOutNames is not None:
            for key in self.fTmpOutNames:
                try:
                    os.remove(self.fTmpOutNames[key])
                except:
                    pass

    def _getChromListAndResolution(self):
        """ Get chromosome list and resolution

        Read input files headers and determine chromosome names and resolution. After call this function, chromosome list can be found in :attr:`HOMERInputHandler.chromList`
        and Resolution can be found in :attr:`HOMERInputHandler.resolution`

        """

        self._openInputFiles()

        # Removing previous header list
        if self.headers is None:
            self.headers = []
        else:
            del self.headers
            self.headers = []

        chroms = []
        self.logger.info(" Getting chromosome list and resolution from Input Files ...")
        for fin in self.fIns:
            if fin is None: continue
            binsize = None

            # Only read first line
            if self.compressType is None:
                line = fin.readline()
            else:
                line = fin.readline().decode()

            line = line.lstrip().rstrip()
            match = re.search('Regions', line)
            header = re.split('\s+', line[match.start():])

            count = 0
            prev_j = None
            step = []
            for m in range(len(header[1:])):
                chrm, j = re.split('-', header[m+1])
                chroms.append( self._getChromName(chrm) )

                if count < 100 and prev_j is not None:
                    ds = abs( int(j) - int(prev_j) )
                    if ds > 0:
                        step.append(ds)
                        count += 1
                prev_j = j

            if binsize is None:
                self.resolution = cmp.binsizeToResolution( int( np.amin(step) ) )
            else:
                if binsize != int( np.amin(step) ):
                    raise AssertionError ('Data Resolution does not match in input files.')

            # Appending header of each file
            self.headers.append(header)

        self.chromList = cmh.sorted_nicely(list(set(chroms)))

        self.logger.info(" Resolution: {0}" .format(self.resolution))
        outline = ' '
        for chrm in self.chromList:
            outline = outline + "\n                              " + chrm
        self.logger.info(" Following chromsomes found in input files: {0}" .format(outline))

    def _getChromName(self, chrm):
        """Generate chromosome name
        Sometimes, input file only contains chromosome names. This function change number to name by prefixing it with ``chr``

        """
        if re.search('(chr)|(Chr)', chrm) == None:
            return 'chr' + chrm
        else:
            return chrm

    def _ProcessInputs(self):
        """Read input files and save map to temporary text file

        This function reads input files and save data temporarily in output
        text files. Name of these temporary files are listed
        in :attr:`HOMERInputHandler.fTmpOutNames` .

        """

        self._generateTemporaryOutputFiles()

        try:
            for n in range(len(self.fIns)):

                # When a file is not found in compressed file, file object could be none
                if self.fIns[n] is None:    continue

                self.logger.info(" Reading [{0}] file ..." .format(self.inputFileList[n]))

                while(1):
                    if self.compressType is None:
                        line = self.fIns[n].readline()
                    else:
                        line = self.fIns[n].readline().decode()

                    if not line:    break

                    line = line.lstrip().rstrip()
                    temp = re.split('\s+', line)

                    # Header is taken from available header list. Also, first line is already read from each input file
                    chrm1, i = re.split('-', temp[1])
                    #print(len(temp[2:]), chrm1, i, len(header[1:]))
                    for m in range(len(self.headers[n][1:])):
                        chrm2, j = re.split('-', self.headers[n][m+1])
                        if chrm1 == chrm2:
                            val = float(temp[m+2])
                            if val != 0:
                                self.fTmpOut[self._getChromName(chrm1)].write('{0}\t{1}\t{2}\n' .format(i, j, val))

                self.logger.info("           ... Finished reading [{0}] file ..." .format(self.inputFileList[n]))

        except (KeyboardInterrupt, SystemExit) as e:
            self._removeTemporaryOutputFiles()
            raise e

        self._closeInputFiles()
        self._closeTemporaryOutputFiles()

    def save_ccmaps(self, outdir, suffix=None):
        """Import and save ccmap file

        Read input files, save data temporarily in text file for each chromosome and import these data to native ccmap format using
        :class:`CooMatrixHandler` class.

        .. note::
            * Output file names will be automatically generated as ``<chromosome>_<resolution>.ccmap`` format. e.g. ``chr12_10kb.ccmap``.
            * A suffix can be added to all files as ``<chromosome>_<resolution>_<suffix>.ccmap`` format. e.g. if ``suffix='_RawObserved'``, file name is ``chr12_10kb_RawObserved.ccmap``.

        Parameters
        ----------
        outdir : str
            Path to directory where all ccmaps have to be saved
        suffix : str
            Any suffix to file name
        """
        self._ProcessInputs()

        outputFiles = []
        inputFiles = []
        try:
            for i in range(len(self.chromList)):

                if suffix is None:
                    outFileName = self.chromList[i] + '_' + self.resolution + '.ccmap'
                else:
                    outFileName = self.chromList[i] + '_' + self.resolution + '_' + suffix + '.ccmap'

                outFile = os.path.join(outdir, outFileName)
                outputFiles.append(outFile)

                inputFiles.append(self.fTmpOutNames[self.chromList[i]])

            reader = CooMatrixHandler(inputFiles, resolution=self.resolution)
            reader.save_ccmaps(outputFiles,
                                xlabels=self.chromList,
                                ylabels=self.chromList)
            self.logger.info(" Saved {0} files." .format(outputFiles))

        except (KeyboardInterrupt, SystemExit) as e:
            self._removeTemporaryOutputFiles()
            raise e

    def save_gcmap(self, outputFile, coarsingMethod='sum', compression='lzf'):
        """To Save all Hi-C maps as a gcmap file

        This function reads input files one by one and save it as a ``.gcmap``
        file.

        Parameters
        ----------
        outputFile : str
            Name of a output gcmap file.
        coarsingMethod : str
            Method of downsampling. Three accepted methods are ``sum``: sum all
            values, ``mean``: Average of all values and ``max``: Maximum of
            all values.
        compression : str
            Compression method. Presently allowed : ``lzf`` for LZF compression
            and ``gzip`` for GZIP compression.

        """

        self._ProcessInputs()

        inputFiles = []
        try:
            for i in range(len(self.chromList)):
                inputFiles.append(self.fTmpOutNames[self.chromList[i]])

            reader = CooMatrixHandler(inputFiles, resolution=self.resolution)
            reader.save_gcmap(outputFile,
                                xlabels=self.chromList,
                                ylabels=self.chromList,
                                coarsingMethod=coarsingMethod,
                                compression=compression)

        except (KeyboardInterrupt, SystemExit) as e:
            self._removeTemporaryOutputFiles()
            raise e

class BinsNContactFilesHandler:
    """To import Hi-C map from bin and contact file in list format

    These types of files are appeared in following GEO data:
        * http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE61471
        * http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE34453

    Parameters
    ----------
    binFile : str
        Bin file
    contactFile : str
        Contact file in list format
    workDir : str
        Directory where temporary files will be stored. If it is not provided, this value is taken from configuration file.

    Attributes
    ----------
    binFile : str
        Bin file
    contactFile : str
        Contact file in list format
    ChromSize : dict
        Dictionary of Chromosome size
    ChromBinsInfo : dict
        Dictionary of min (start) and max (end) bin number for each Chromosome
    npyBinFileList : dict
        Dictionary containing tuple (memmap stream, Temporary numpy array file name)
    binsize : int
        Bin size of Hi-C map
    ccmaps : dict
        Dictionary of CCMAP instances for each Chromosome


    """
    def __init__(self, binFile, contactFile, workDir=None, logHandler=None):
        self.binFile = binFile
        self.contactFile = contactFile

        self.ChromSize = None           # Store Chromosome size
        self.ChromBinsInfo = None       # Store max and min Bins for each Chromosome
        self.npyBinFileList = None      # Temporary numpy array dict containing tuple (npy, filename)

        self.binsize = None

        self.ccmaps = None            # ccmap list for chromosomes

        if workDir is not None:
            self.workDir = workDir
        else:
            self.workDir = config['Dirs']['WorkingDirectory']

        # logger
        self.logHandler = logHandler
        self.logger = logging.getLogger('BinsNContactFilesHandler')
        if logHandler is not None:
            self.logger.propagate = False
            self.logger.addHandler(logHandler)
        self.logger.setLevel(logging.INFO)

    def __del__(self):
        self._removeTempFiles()
        if self.logHandler is not None:
            self.logger.removeHandler( self.logHandler )

    def _removeTempFiles(self):
        """Remove temporary numpy array files
        """
        if self.npyBinFileList is not None:
            for key in self.npyBinFileList:
                try:
                    self.npyBinFileList[key][0].close()
                except:
                    pass

                try:
                    self.logger.info(' Removing temporary numpy array file [{0}] for {1} ...' .format(self.npyBinFileList[key][1], key))
                    os.remove(self.npyBinFileList[key][1])
                except:
                    pass

    def _getChromName(self, chrm):
        """Generate chromosome name
        Sometimes, input file only contains chromosome names. This function change number to name by prefixing it with ``chr``

        """
        if re.search('(chr)|(Chr)', chrm) == None:
            return 'chr' + chrm
        else:
            return chrm

    def _printChromosomeInfo(self):
        """Print information about chromosomes
        """

        outline = '  Chromosome Size: \n'
        for key in self.ChromSize:
            outline = outline + '{0:>50} : {1}\n' .format(key, self.ChromSize[key])
        self.logger.info(outline)

        if self.ChromBinsInfo is not None:
            outline = '  Chromosome Bins info:\n'
            for key in self.ChromBinsInfo:
                outline = outline +  '{0:>55}: {1}\n' .format(key, self.ChromBinsInfo[key])
            self.logger.info(outline)

        if self.npyBinFileList is not None:
            outline = '   Temporary array size info:\n'
            for key in self.npyBinFileList:
                outline = outline + '{0:>70}: {1}\n' .format( key, self.npyBinFileList[key][0].shape[0] )
            self.logger.info(outline)

    def _printHiCmapInfo(self):
        """Print information about Hi-C maps
        """
        if self.ccmaps is not None:
            outline = '   Hi-C Maps Summary:\n'
            outline = outline + '{0:>60}\tSize\t\tMax. \tMin. \n'.format('Chromosome')
            for key in self.ccmaps:
                self.ccmaps[key].make_readable()
                outline = outline + '{0:>60}\t{1}\t{2}\t{3}\n' .format(key, self.ccmaps[key].shape, self.ccmaps[key].maxvalue, self.ccmaps[key].minvalue)
            self.logger.info(outline)

    def _readBinFile(self):
        """Read bin file

        Reads bin file and generates ``ChromSize`` and ``ChromBinsInfo`` dictionary
        """

        fin = open(self.binFile, 'r')

        ChromTitle = 'dummy'
        PrevChromTitle = 'dummy'

        for line in fin:
            line = line.lstrip().rstrip()

            # Do not read a line that start with word
            if re.match('\d+', line) == None:
                continue

            temp = re.split('\s+', line)
            s0 = int(temp[2])
            s1 = int(temp[3])

            if self.binsize is not None:
                if self.binsize != s1-s0:
                    raise ValueError('Length of bins are not constant in files. Exiting.')
            else:
                self.binsize = s1-s0

            PrevChromTitle = ChromTitle
            ChromTitle = temp[1]

            if ChromTitle !=  PrevChromTitle:
                if self.ChromSize is None:
                    self.ChromSize = dict()

                if self.ChromBinsInfo is None:
                    self.ChromBinsInfo = dict()

                if ChromTitle not in self.ChromBinsInfo:
                    self.ChromBinsInfo[ChromTitle] = { 'min': int(temp[0]), 'max':  None }

                self.ChromSize[ChromTitle] = s1
                self.ChromBinsInfo[ChromTitle]['max'] = int(temp[0])

            else:
                self.ChromSize[ChromTitle] = s1
                self.ChromBinsInfo[ChromTitle]['max'] = int(temp[0])

        # Print information about chromosome
        self._printChromosomeInfo()

        # Generating temporary numpy array files
        for key in self.ChromSize:
            if self.npyBinFileList is None:
                self.npyBinFileList = dict()

            name = cmp.name_generator()
            (fout, fname) = tempfile.mkstemp(suffix='.npy', prefix=key+'_', text=False, dir=self.workDir)
            os.close(fout)
            self.logger.info(' Generating temporary numpy array file [{0}] for {1} ...' .format(fname, key))

            size = int(self.ChromSize[key]/self.binsize)+1
            np.save(fname, np.zeros(size, dtype=np.int))
            npy = np.load(fname, mmap_mode='r+')
            self.npyBinFileList[key] = (npy, fname)

            self.logger.info(' Finished. \n' .format(fname, key))


        # Re-read file to store bin index
        fin.seek(0)
        for line in fin:
            line = line.lstrip().rstrip()

            # Do not read a lin that start with word
            if re.match('\d+', line) == None:
                continue

            temp = re.split('\s+', line)
            ChromTitle = temp[1]

            idx = int(int(temp[3])/self.binsize)
            self.npyBinFileList[ChromTitle][0][idx] = int(temp[0])

        fin.close()

    def _readContactFile(self):
        """Read contact file and generate CCMAP instances
        """
        fin = open(self.contactFile, 'r')

        ChromTitle = 'dummy'
        PrevChromTitle = 'dummy'

        self.logger.info(' Reading contact file ...\n')
        i, j, value = [], [], []
        for line in fin:
            line = line.lstrip().rstrip()

            # Do not read a lin that start with word
            if re.match('\d+', line) == None:
                continue

            temp = re.split('\s+', line)
            iBin = int(temp[0])
            jBin = int(temp[1])
            c = float(temp[3])

            for key in self.ChromBinsInfo:
                if (min(iBin, jBin) >= self.ChromBinsInfo[key]['min']) and (max(iBin, jBin) <=  self.ChromBinsInfo[key]['max']):
                    idx = np.nonzero( self.npyBinFileList[key][0] == iBin )
                    jdx = np.nonzero( self.npyBinFileList[key][0] == jBin )
                    i.append(idx[0][0]*self.binsize)
                    j.append(jdx[0][0]*self.binsize)
                    value.append(c)

                    ChromTitle = key

            if ChromTitle != PrevChromTitle and PrevChromTitle != 'dummy':
                self.logger.info(' \tGenerating Hi-C Map for [{0}] ... \n' .format(PrevChromTitle))
                ccmap = gen_map_from_locations_value(i, j, value, resolution=cmp.binsizeToResolution(self.binsize), workDir=self.workDir, mapType='intra')
                ccmap.xlabel = self._getChromName(PrevChromTitle)
                ccmap.ylabel = self._getChromName(PrevChromTitle)

                if self.ccmaps is None:
                    self.ccmaps = dict()

                self.ccmaps[PrevChromTitle] = ccmap
                i, j, value = [], [], []
                self.logger.info(' Finished\n' .format(PrevChromTitle))


            PrevChromTitle = ChromTitle


        # Last One
        self.logger.info(' \tGenerating Hi-C Map for [{0}] ... ' .format(ChromTitle))
        ccmap = gen_map_from_locations_value(i, j, value, resolution=cmp.binsizeToResolution(self.binsize),  workDir=self.workDir, mapType='intra')
        ccmap.xlabel = self._getChromName(PrevChromTitle)
        ccmap.ylabel = self._getChromName(PrevChromTitle)

        self.ccmaps[ChromTitle] = ccmap
        i, j, value = [], [], []
        self.logger.info(' Finished\n' .format(ChromTitle))

        self.logger.info(' Finished reading contact file.\n')

        # Print Hi-C Map information summary
        self._printHiCmapInfo()

    def _genHiCMaps(self):
        """Read both bin and contact file and generated CCMAP
        """
        try:
            self._readBinFile()
            self._readContactFile()
        except (SystemExit, KeyboardInterrupt) as e:
            self._removeTempFiles()
            if self.ccmaps is not None:
                for key in self.ccmaps:
                    del self.ccmaps[key]
            raise e

    def save_ccmaps(self, outdir, suffix=None):
        """Import and save ccmap file

        Read input files, save data temporarily for each chromosome and import these data to native ccmap format.

        ..note::
            * Output file names will be automatically generated as ``<chromosome>_<resolution>.ccmap`` format. e.g. ``chr12_10kb.ccmap``.
            * A suffix can be added to all files as ``<chromosome>_<resolution><suffix>.ccmap`` format. e.g. if ``suffix='_RawObserved'``, file name is ``chr12_10kb_RawObserved.ccmap``.

        Parameters
        ----------
        outdir : str
            Path to directory where all ccmaps have to be saved
        suffix : str
            Any suffix to file name
        """
        if self.ccmaps is None:
            self._genHiCMaps()

        resolution = cmp.binsizeToResolution(self.binsize)
        for key in self.ccmaps:
            if suffix is None:
                outFileName = self._getChromName(key) + '_' + resolution + '.ccmap'
            else:
                outFileName = self._getChromName(key) + '_' + resolution + suffix + '.ccmap'

            fullOutPath = os.path.join(outdir, outFileName)

            cmp.save_ccmap(self.ccmaps[key], fullOutPath, compress=True)

    def save_gcmap(self, outputFile, coarsingMethod='sum', compression='lzf'):
        """To Save all Hi-C maps as a gcmap file

        This function reads input files one by one and save it as a ``.gcmap``
        file.

        Parameters
        ----------
        outputFile : str
            Name of a output gcmap file.
        coarsingMethod : str
            Method of downsampling. Three accepted methods are ``sum``: sum all
            values, ``mean``: Average of all values and ``max``: Maximum of
            all values.
        compression : str
            Compression method. Presently allowed : ``lzf`` for LZF compression
            and ``gzip`` for GZIP compression.

        """

        if self.ccmaps is None:
            self._genHiCMaps()

        resolution = cmp.binsizeToResolution(self.binsize)
        for key in self.ccmaps:
            gmp.addCCMap2GCMap(self.ccmaps[key], outputFile,
                                compression=compression,
                                coarsingMethod=coarsingMethod)


def gen_map_from_locations_value(i, j, value, resolution=None, mapType='intra', workDir=None, logHandler=None):
    """To generate CCMAP object from three lists -- i, j, value

    Parameters
    ----------
    i : list[int]
        List of first location from each pair
    j : list[int]
        List of second location from each pair
    resolution : str
        Resolution of Hi-C map
    mapType : str
        Hi-C map type: ``intra`` or ``inter`` chromosomal map
    value : list[float]
        List of values for respective location
    workDir : str
        Directory where temporary files will be stored. If it is not provided, this value is taken from configuration file.

    Returns
    -------
    ccMapObj : :class:`hiCMapAnalyze.HiCMapMain.CCMAP`
    	A CCMAP object

    """

    # logger
    logHandler = logHandler
    logger = logging.getLogger('genMapFromLists')
    if logHandler is not None:
        logger.propagate = False
        logger.addHandler(logHandler)
    logger.setLevel(logging.INFO)

    # set working directory
    if workDir is None:
        workDir = config['Dirs']['WorkingDirectory']

    ccMapObj = cmp.CCMAP()

    maxLi = np.amax(i)
    maxLj = np.amax(j)

    maxL = np.amax([i, j])

    if resolution is not None:
        ccMapObj.binsize = cmp.resolutionToBinsize(resolution)
    else:
        step = []
        max_count = len(i)
        for n in range(1, 100):
            ds = abs(i[n] - i[n-1])
            if ds > 0:
                step.append(ds)
            ds = abs(j[n] - j[n-1])
            if ds > 0:
                step.append(ds)

            if n > max_count:
                break

        ccMapObj.binsize = int( np.amin(step) )

    logger.info(" Total number of data in input file: {0}" .format(len(i)))
    if mapType == 'intra':
    	minL = np.amin([i, j])
    	logger.info("Minimum base-pair: {0} and Maximum base-pair: {1} are present in input data" .format(minL, maxL))

    	xticks = np.arange(0, maxL + ccMapObj.binsize, ccMapObj.binsize)
    	yticks = np.arange(0, maxL + ccMapObj.binsize, ccMapObj.binsize)
    	ccMapObj.xticks = [0, maxL + ccMapObj.binsize]
    	ccMapObj.yticks = [0, maxL + ccMapObj.binsize]
    else:
    	xticks = np.arange(0, maxLi + ccMapObj.binsize, ccMapObj.binsize)
    	yticks = np.arange(0, maxLj + ccMapObj.binsize, ccMapObj.binsize)
    	ccMapObj.xticks = [0, maxLi + ccMapObj.binsize]
    	ccMapObj.yticks = [0, maxLj + ccMapObj.binsize]

    xdlen = len(xticks)
    ydlen = len(yticks)
    ccMapObj.shape = (xdlen, ydlen)

    # Creating a file name for binary numpy array on disk
    ccMapObj.gen_matrix_file(workDir=workDir)

    # Creating a file for binary numpy array on disk
    ccMapObj.make_writable()

    logger.info("Shape of overall map: ({0}, {0})\n" .format(xdlen, ydlen))

    # print(len(i), len(value), ccMapObj.binsize, ccMapObj.shape)
    try:
        for n in range(len(i)):
            ni = int( i[n]/ccMapObj.binsize )
            nj = int( j[n]/ccMapObj.binsize )

            # In case if two observation is present for same pair of locations. take average of them
            if ccMapObj.matrix[ni][nj] != 0:
                ccMapObj.matrix[ni][nj] = (ccMapObj.matrix[ni][nj] + value[n])/2
            else:
                ccMapObj.matrix[ni][nj] = value[n]

            if mapType == 'intra':
                ccMapObj.matrix[nj][ni] = ccMapObj.matrix[ni][nj]

            if (ccMapObj.maxvalue < value[n]):
            	ccMapObj.maxvalue = value[n]
    except (KeyboardInterrupt, SystemExit) as e:
    	del ccMapObj
    	raise e

    # Get minimum value after zero
    ma = np.ma.masked_equal(ccMapObj.matrix, 0.0, copy=False)
    ccMapObj.minvalue = ma.min()
    del ma

    ccMapObj.make_unreadable()

    if logHandler is not None:
        logger.removeHandler( logHandler )

    return ccMapObj



__all__ = [ 'ccmap',
            'gcmap',
            'importer',
            'ccmapHelpers',
            'cmstats',
            'normalizer',
            'normalizeKnightRuiz',
            'genomicsDataHandler',
            'TadFinder']



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os
import sys
import re
import argparse

import gcMapExplorer.lib as gmlib

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

description = \
""" Normalization by Iterative Correction method
===============================================================

This method normalize the raw contact map by removing biases from experimental procedure.
For more details, see this publication: http://dx.doi.org/10.1038/nmeth.2148


Please cite:
Imakaev et al. (2012), Iterative correction of Hi-C data reveals hallmarks of chromosome organization.
Nature Methods 9, 999–1003 (2012). http://dx.doi.org/10.1038/nmeth.2148

================================================================
"""

inputFileHelp = \
""" Input ccmap or gcmap file.

"""

inputFileFormatHelp = \
""" Input format: 'ccmap' or 'gcmap'.

"""


outputFileHelp = \
""" Output ccmap or gcmap file.

When input file is ccmap, ouput file can be gcmap. However, when a input file
is gcmap, output file will be only in gcmap.

"""

outputFileFormatHelp = \
""" Input format: 'ccmap' or 'gcmap'.

When input file is ccmap, ouput file can be gcmap. However, when a input file
is gcmap, output file will be only in gcmap.

"""

tolHelp = \
""" Tolerance for matrix balancing.
Smaller tolreance increases accuracy in sums of rows and columns.

"""

iterationHelp = \
""" Number of iteration to stop the normalization.

"""

percentile_thershold_no_data_help = \
""" It can be used to filter the map, where rows/columns with largest numbers
of missing data can be discarded. Its value should be between 1 and 100.
This options discard the rows and columns which are above this percentile.
For example: if this value is 99, those rows or columns will be discarded which
contains larger than number of zeros (missing data) at 99 percentile.

To calculate percentile, all blank rows are removed, then in all rows, number
of zeros are counted. Afterwards, number of zeros at input percentile is
obtained. In next step, if a row contain number of zeros larger than this
percentile value, the whole row and column is assigned to have missing data.
This percentile indicates highest numbers of zeros (missing data) in given
rows/columns.

"""

thershold_data_occup_help = \
""" It can be used to filter the map, where rows/columns with largest numbers
of missing data can be discarded.This ratio is:
  (number of bins with data) / (total number of bins in the given row/column)

For example: if -tdo = 0.8, then all rows containing more than 20%% of
missing data will be discarded.

"""

citation = \
"""
Please Cite
===========
Imakaev et al. (2012), Iterative correction of Hi-C data reveals hallmarks of chromosome organization.
Nature Methods 9, 999–1003. http://dx.doi.org/10.1038/nmeth.2148

"""


def main():

    # Construct command line arguments and parsed it
    parser, args = parseArguments()

    # check input file exist
    if args.inputFile is not None:
        checkFileExist(args.inputFile, parser)
    else:
        msg = 'No input file!!!'
        showErrorAndExit(parser, msg)

    # check input file format provided
    if args.inputFileFormat is None:
        msg = 'No input file format given !!!'
        showErrorAndExit(parser, msg)

    # check output file provided
    if args.outputFile is None:
        msg = 'No output file name given !!!'
        showErrorAndExit(parser, msg)

    # check output file format provided
    if args.outputFileFormat is None:
        msg = 'No output file format given !!!'
        showErrorAndExit(parser, msg)

    if args.outputFileFormat == 'ccmap' and args.inputFileFormat == 'gcmap':
        msg = 'With gcmap input format, output format should be gcmap format.'
        showErrorAndExit(parser, msg)

    if args.percentile_thershold_no_data is not None and args.thershold_data_occup is not None:
        msg = 'Both Percentile and Fraction filters cannot be used simultaneously!!'
        showErrorAndExit(parser, msg)

    if args.outputFileFormat == 'ccmap' and args.inputFileFormat == 'ccmap':
        gmlib.normalizer.normalizeCCMapByIC(args.inputFile, tol=args.tol, iteration=args.iteration,
                                            outFile=args.outputFile,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)

    if args.outputFileFormat == 'gcmap' and args.inputFileFormat == 'ccmap':
        norm_ccmap = gmlib.normalizer.normalizeCCMapByIC(args.inputFile, tol=args.tol,  iteration=args.iteration,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)
        gmlib.gcmap.addCCMap2GCMap(norm_ccmap, args.outputFile, compression=args.compression)


    if args.outputFileFormat == 'gcmap' and args.inputFileFormat == 'gcmap':
        gmlib.normalizer.normalizeGCMapByIC(args.inputFile, args.outputFile,
                                            tol=args.tol,  iteration=args.iteration,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)

    print(citation)

def parseArguments():
    parser = argparse.ArgumentParser(
                prog='gcMapExplorer normIC',
                description=description,
                formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-i', '--input', action='store',
                        metavar='input.gcmap',
                        dest='inputFile', help=inputFileHelp)

    parser.add_argument('-fi', '--format-input', action='store',
                        metavar='gcmap',
                        choices=['gcmap', 'ccmap', 'hicmap'],
                        dest='inputFileFormat', help=inputFileFormatHelp)

    parser.add_argument('-o', '--output', action='store',
                        metavar='output.gcmap',
                        dest='outputFile', help=outputFileHelp)

    parser.add_argument('-fo', '--format-output', action='store',
                        metavar='gcmap',
                        choices=['gcmap', 'ccmap'],
                        dest='outputFileFormat', help=outputFileFormatHelp)

    parser.add_argument('-t', '--tolerance', action='store',
                        dest='tol', metavar=1e-4,
                        type = float, default=1e-4,
                        help=tolHelp)

    parser.add_argument('-c', '--iteration', action='store',
                        dest='iteration', metavar=500,
                        type = int, default=500,
                        help=iterationHelp)

    parser.add_argument('-ptnd', '--percentile-thershold-no-data', action='store',
                        dest='percentile_thershold_no_data', metavar=99,
                        type = float,
                        help=percentile_thershold_no_data_help)

    parser.add_argument('-tdo', '--thershold-data-occupancy', action='store',
                        dest='thershold_data_occup', metavar=0.8,
                        type = float,
                        help=thershold_data_occup_help)

    parser.add_argument('-cmeth', '--compression-method', action='store',
                        dest='compression', metavar='lzf',
                        choices=['lzf', 'gzip'], default='lzf',
                        help='Data compression method for output gcmap file.\n')

    parser.add_argument('-wd', '--work-dir', action='store', dest='workDir',
                        default=config['Dirs']['WorkingDirectory'],
                        metavar=config['Dirs']['WorkingDirectory'],
                        help='Directory where temporary files will be stored.')

    idx = sys.argv.index("normIC")+1
    args = parser.parse_args(args=sys.argv[idx:])

    return parser, args


def checkFileExist(filename, parser):
    if not os.path.isfile(filename):
        parser.print_help()
        print("\n===== ERROR =======")
        print("File: {0} does not exist !!!\n".format(filename))
        print("See Usage Above!!!")
        sys.exit(-1)


def showErrorAndExit(parser, message):
    parser.print_help()
    print("\n===== ERROR =======")
    print(message)
    print("See Usage Above!!!")
    sys.exit(-1)



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import logging
import os

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.uic import loadUiType


# Determine absolute path to UIs directory. Relative path from this directory does not work.
DirToThisScript = os.path.dirname(os.path.abspath(__file__))
PathToUIs = os.path.join(DirToThisScript, 'UIs')

## To divert log output into QPlainTextWidget
class pyQtLogHandler(QObject, logging.Handler):
    messageEmitted = pyqtSignal( str )
    def __init__(self):
        super().__init__()
        self.setFormatter(logging.Formatter('%(levelname)s:%(name)s:%(message)s'))

    def emit(self, record):
        msg = self.format(record)
        self.messageEmitted.emit(msg)

class qtThread(QThread):
    resultReady = pyqtSignal()

    def __init__(self, parent=None, target=None, args=(), kwargs=None):
        super(qtThread, self).__init__(parent)
        self.results = None
        self._target = target
        self._args = args
        self._kwargs = kwargs


    def callRun(self):
        try:
            if self._target:
                if self._kwargs is not None:
                    self.results = self._target(*self._args, **self._kwargs)
                else:
                    self.results = self._target(*self._args)
        finally:
            # Avoid a refcycle if the thread is running a function with
            # an argument that has a member that points to the thread.
            del self._target, self._args, self._kwargs

        self.resultReady.emit()

    def run(self):
        self.callRun()


def qThreadTerminated():
    currentThread = qtThread.currentThread()
    if currentThread is not None:
        if currentThread.isInterruptionRequested():
            return True
        else:
            return False
    else:
        return False


def getSelectedRowColumnFromTable(table):
    # Total number of row
    r = table.rowCount()
    c = table.columnCount()

    # Determine which cell of row is selected
    selectedRow = None
    selectedCol = None
    for i in range(r):
        for j in range(c):
            if table.item(i, j) is None:
                table.setItem( i, j, QTableWidgetItem(0) )

            if table.item(i, j).isSelected():
                selectedRow = i
                selectedCol = j
                break

    return selectedRow, selectedCol

def checkFileExist(lineEdit, qwidget):
    if not lineEdit.text(): return
    filename = str( lineEdit.text() )
    if not os.path.isfile( filename ):
        msg = "[ {0} ] \n Not found !!!".format(filename)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.undo()
        lineEdit.setFocus()

def checkDirExist(lineEdit, qwidget):
    if not lineEdit.text(): return
    dirname = str( lineEdit.text() )
    if not os.path.isdir( dirname ):
        msg = "[ {0} ] \n Not found !!!".format(dirname)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.undo()
        lineEdit.setFocus()

def showWarningMessageBox(msg, qwidget):
    msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Ok, qwidget)
    msgBox.exec_()
    msgBox.close()

def constrainValueInLineEdit(qwidget, lineEdit, minvalue, maxvalue,):
    """ Whether value in a lineEdit is within limit.

    It checks whether value in a lineEdit is between the limit, otherwise
    throw a warning to user.

    """
    value = float(lineEdit.text())

    if not value:   return

    if value > maxvalue or value < minvalue:
        msg = "Value should be between {0} and {1} !!!".format(minvalue, maxvalue)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.selectAll()
        lineEdit.setFocus()

def getFileExtension(name):
    return os.path.splitext(name)[1]
